Type: Package
Package: INLAtools
Title: Functionalities for the 'INLA' Package
Version: 0.1.4
Authors@R: c(
    person("Elias", "Teixeira Krainski", , "elias.krainski@kaust.edu.sa", role = c("cre", "aut", "cph"),
           comment = c(ORCID = "0000-0002-7063-2615")),
    person("Finn", "Lindgren", , "finn.lindgren@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0002-5833-2011")),
    person("Haavard", "Rue’", role = "aut",
           comment = c(ORCID = "0000-0002-0222-1881"))
  )
Maintainer: Elias Teixeira Krainski <elias.krainski@kaust.edu.sa>
Description: Contain code to work with a C struct, in short cgeneric, to
    define a Gaussian Markov random (GMRF) model.  The cgeneric contain
    code to specify GMRF elements such as the graph and the precision
    matrix, and also the initial and prior for its parameters, useful for
    model inference.  It can be accessed from a C program and is the
    recommended way to implement new GMRF models in the 'INLA' package
    (<https://www.r-inla.org>).  The 'INLAtools' implement functions to
    evaluate each one of the model specifications from R.  The implemented
    functionalities leverage the use of 'cgeneric' models and provide a
    way to debug the code as well to work with the prior for the model
    parameters and to sample from it.  The `generic0` can be used to
    implement intrinsic models with the scaling as proposed in Sørbye &
    Rue (2014) <doi:10.1016/j.spasta.2013.06.004>, and the required
    constraints.  A very useful functionality is the Kronecker product
    method that creates a new model from multiple cgeneric models.  It
    also works with the rgeneric, the R version of the cgeneric intended
    to easy try implementation of new GMRF models.  The Kronecker between
    two cgeneric models where each one needs a constraint, such as
    spatio-temporal intrinsic interaction models, the needed constraints
    are automatically set.
License: GPL (>= 2)
URL: https://github.com/eliaskrainski/INLAtools
BugReports: https://github.com/eliaskrainski/INLAtools/issues
Depends: Matrix, R (>= 4.3)
Imports: methods, utils
Encoding: UTF-8
NeedsCompilation: yes
RoxygenNote: 7.3.3
Packaged: 2026-05-04 13:12:02 UTC; eliask
Author: Elias Teixeira Krainski [cre, aut, cph] (ORCID:
    <https://orcid.org/0000-0002-7063-2615>),
  Finn Lindgren [aut] (ORCID: <https://orcid.org/0000-0002-5833-2011>),
  Haavard Rue’ [aut] (ORCID: <https://orcid.org/0000-0002-0222-1881>)
Repository: CRAN
Date/Publication: 2026-05-04 14:50:13 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-05-04 16:59:21 UTC; unix
Archs: INLAtools.so.dSYM

Type: Package
Package: INLAtools
Title: Functionalities for the 'INLA' Package
Version: 0.1.4
Authors@R: c(
    person("Elias", "Teixeira Krainski", , "elias.krainski@kaust.edu.sa", role = c("cre", "aut", "cph"),
           comment = c(ORCID = "0000-0002-7063-2615")),
    person("Finn", "Lindgren", , "finn.lindgren@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0002-5833-2011")),
    person("Haavard", "Rue’", role = "aut",
           comment = c(ORCID = "0000-0002-0222-1881"))
  )
Maintainer: Elias Teixeira Krainski <elias.krainski@kaust.edu.sa>
Description: Contain code to work with a C struct, in short cgeneric, to
    define a Gaussian Markov random (GMRF) model.  The cgeneric contain
    code to specify GMRF elements such as the graph and the precision
    matrix, and also the initial and prior for its parameters, useful for
    model inference.  It can be accessed from a C program and is the
    recommended way to implement new GMRF models in the 'INLA' package
    (<https://www.r-inla.org>).  The 'INLAtools' implement functions to
    evaluate each one of the model specifications from R.  The implemented
    functionalities leverage the use of 'cgeneric' models and provide a
    way to debug the code as well to work with the prior for the model
    parameters and to sample from it.  The `generic0` can be used to
    implement intrinsic models with the scaling as proposed in Sørbye &
    Rue (2014) <doi:10.1016/j.spasta.2013.06.004>, and the required
    constraints.  A very useful functionality is the Kronecker product
    method that creates a new model from multiple cgeneric models.  It
    also works with the rgeneric, the R version of the cgeneric intended
    to easy try implementation of new GMRF models.  The Kronecker between
    two cgeneric models where each one needs a constraint, such as
    spatio-temporal intrinsic interaction models, the needed constraints
    are automatically set.
License: GPL (>= 2)
URL: https://github.com/eliaskrainski/INLAtools
BugReports: https://github.com/eliaskrainski/INLAtools/issues
Depends: Matrix, R (>= 4.3)
Imports: methods, utils
Encoding: UTF-8
NeedsCompilation: yes
RoxygenNote: 7.3.3
Packaged: 2026-05-04 13:12:02 UTC; eliask
Author: Elias Teixeira Krainski [cre, aut, cph] (ORCID:
    <https://orcid.org/0000-0002-7063-2615>),
  Finn Lindgren [aut] (ORCID: <https://orcid.org/0000-0002-5833-2011>),
  Haavard Rue’ [aut] (ORCID: <https://orcid.org/0000-0002-0222-1881>)
Repository: CRAN
Date/Publication: 2026-05-04 14:50:13 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-04 22:46:53 UTC; unix
Archs: INLAtools.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(cgeneric,"function")
S3method(cgeneric,cgeneric)
S3method(cgeneric,character)
S3method(cgeneric,inla.cgeneric)
S3method(is.zero,Matrix)
S3method(is.zero,default)
S3method(is.zero,matrix)
S3method(plot,cgeneric)
S3method(plot,rgeneric)
S3method(print,cgeneric)
S3method(print,rgeneric)
S3method(rgeneric,default)
S3method(rgeneric,inla.rgeneric)
S3method(rgeneric,rgeneric)
S3method(summary,cgeneric)
S3method(summary,rgeneric)
export(Sparse)
export(cgeneric)
export(cgenericBuilder)
export(cgeneric_Q)
export(cgeneric_generic0)
export(cgeneric_graph)
export(cgeneric_iid)
export(cgeneric_initial)
export(cgeneric_mu)
export(cgeneric_prior)
export(cgeneric_shlib_path)
export(findGetFunction)
export(inlaQ)
export(is.zero)
export(multi_generic_model)
export(packageCheck)
export(pcParamCheck)
export(pclrange)
export(pcrange)
export(rgeneric)
export(rgeneric_Q)
export(rgeneric_graph)
export(rgeneric_initial)
export(rgeneric_mu)
export(rgeneric_prior)
export(upperPadding)
exportMethods(kronecker)
importFrom(Matrix,Diagonal)
importFrom(Matrix,as.matrix)
importFrom(Matrix,diag)
importFrom(Matrix,sparseMatrix)
importFrom(graphics,image)
importFrom(methods,as)
importFrom(methods,is)
importFrom(stats,vcov)
importFrom(utils,str)
useDynLib(INLAtools)

library(INLAtools)
R1 <- Matrix(crossprod(diff(diag(4))))
R1
m1 <- cgeneric("generic0", R = R1, param = c(1, NA), scale = FALSE)
R2 <- Matrix(crossprod(diff(diag(3))))
R2
m2 <- cgeneric("generic0", R = R2, param = c(1, NA), scale = FALSE)
m3 <- cgeneric("iid", n = 2, param = c(1, 0.5))
cgeneric_Q(m3, theta = 0.0)
multi123 <- multi_generic_model(
  list(m1 = m1, m2 = m2, m3 = m3)
)
R321 <- Sparse(kronecker(kronecker(Diagonal(2),R2),R1))
R321
all.equal(R321, Sparse(cgeneric_Q(multi123, theta = 0.0)))
if(!is.na(packageCheck("inlabru", "2.13.0.9005"))) {
  print(multi123$mapper)
}

library(INLAtools)
old.par <- par(no.readonly = TRUE)

## Setting the prior parameters
prior.par <- c(1, 0.5) # P(sigma > 1) = 0.5
cmodel <- cgeneric(
  model = "iid", n = 10,
  param = prior.par)

## prior summaries: sigma and log-precision
(lamb <- -log(prior.par[2])/prior.par[1])
(smedian <- qexp(0.5, lamb))
(smean <- 1/lamb)

## mode: at the minimum of - log-prior
(lpmode <- optimize(function(x)
  -cgeneric_prior(cmodel, theta = x),
  c(-10, 30))$minimum)
## mean: integral of x*f(x)dx
(lpmean <- integrate(function(x)
  exp(cgeneric_prior(cmodel, theta = matrix(x, 1)))*x,
  -10, 30)$value)

## prior visualization: log(precision) and sigma
par(mfrow = c(1, 2))
plot(function(x)
 exp(cgeneric_prior(cmodel, theta = matrix(x, nrow=1))),
  -3, 3, n = 601, xlab = "log-precision",
  ylab = "density")
abline(v = lpmode, lwd = 3, col = 2)
rug(-2*log(smedian), lwd = 3, col = 3)
rug(lpmean, lwd = 3, col = 4)
plot(function(x)
 exp(cgeneric_prior(cmodel,
  theta = matrix(
    -2*log(x),
    nrow = 1))+log(2)-log(x)),
  1/100, 10, n = 1000,
  xlab = expression(sigma),
  ylab = "density")
plot(function(x) dexp(x, lamb),
   1/100, 10, n = 1000,
   add = TRUE, lty = 2, col = 2)
rug(smedian, lwd = 3, col = 3)
rug(smean, lwd = 3, col = 4)

par(old.par)

suppressPackageStartupMessages({
	library(INLAtools)
	library(INLA)
})

R <- Sparse(crossprod(diff(diag(10))))
R

cmodel <- cgeneric("generic0", R = R, param = c(1, 0.01))

cfam <- list(hyper = list(prec = list(initial = 10, fixed = TRUE)))

fit <- inla(formula = y ~ 0 + f(i, model = cmodel), 
	    data = data.frame(y = NA, i = 1:nrow(R)), 
	    control.family = cfam)

stopifnot(isTRUE(fit$ok))

library(INLAtools)

R <- Sparse(crossprod(diff(diag(10))))
R

m <- cgeneric("generic0", R = R,
	      scale = FALSE,
	      param = c(1, 0.01))
m

all.equal(R, Sparse(cgeneric_Q(m, theta = 0)))

cgeneric_graph(m)
cgeneric_prior(m, theta = 0)
cgeneric_prior(m, theta = matrix(-1:1, 1)) ## see ?prior.cgeneric

library(INLAtools)

### Q = \tau (R2 \otimes R1)

## first
(n1 <- nrow(
     G1 <- sparseMatrix(
         i = c(2, 3, 1, 4, 1, 4, 5, 2, 3, 3),
         j = c(1, 1, 2, 2, 3, 3, 3, 4, 4, 5),
         )
 ))
R1 <- Diagonal(n = n1, x = colSums(G1)) - G1
R1

## 2nd 
(n2 <- nrow(
     R2 <- sparseMatrix(
         i = c(1L, 1L, 2L, 2L, 2L, 3L, 3L),
         j = c(1L, 2L, 1L, 2L, 3L, 3L, 2L),
         x = c(2,-1, -1,3,-1, 4, -1))
 ))
R2

## cgeneric models
cg1 <- cgeneric(
    model = "generic0", 
    R = R1,
    scale = FALSE,
    param = c(1, 0.5)) ## P(sigma > 1) = 0.5
cg2 <- cgeneric(
    model = "generic0", 
    R = R2,
    scale = FALSE,
    param = c(1, NA)) ## fix sigma, simga = 1

## Kronecker of cgeneric models 1 and 2
cg12 <- kronecker(cg1, cg2)

all.equal(Sparse(kronecker(R1, R2)),
          Sparse(cgeneric_Q(cg12, theta = 0)))

#' @rdname cgeneric-class
#' @description
#' A GMRF is defined from model parameters \eqn{\theta} that
#' would parametrize a (sparse) precision matrix.
#'
#' The elements of a GMR are:
#'  *  `graph` to define the non-zero precision matrix pattern.
#'  only the upper triangle including the diagonal is needed.
#'  * `Q` vector where the
#'     * first element (N) is the size of the matrix,
#'     * second element (M) is the number of non-zero
#'     elements in the upper part (including) diagonal
#'     * the remaining (M) elements are the actual
#'     precision (upper triangle plus diagonal) elements
#'     whose order shall follow the graph definition.
#'  * `mu` the mean vector,
#'  * `initial` vector with
#'    * first element as the number of the parameters in the model
#'    * remaining elements should be the initials for the model parameters.
#'  * `log.norm.const` log of the normalizing constant.
#'  * `log.prior` log of the prior for the model parameters.
#' @param model object class for what a `cgeneric` method exists.
#' if it is a character, a specific function will be called,
#' for example cgeneric("iid", ...") calls cgeneric_iid(...).
#' @param ... additional arguments passed on to methods
#' It should include
#' `n` and `debug`. For `cgenericBuild` and
#'  `model` as a character string with the name of the
#'  C function and `shlib` as the path to the
#'  shared object containing such function.
#'  If `shlib` is not provided it can be built
#'  using `cgeneric_shlib_path` for the
#'  `package` (character with the R package containing it),
#'  in which `useINLAprecomp` (logical) indicates if it is to
#'  use the one contained within `INLA` or in the package.
#'  When using the inlabru package,
#'  one can also provide a `mapper` which will be evaluated by
#'  the bru_get_mapper inlabru's function.
#' @note
#' The `graph` and `Q` non-zero pattern should match,
#' its elements should be ordered by row,
#'and only its upper part stored.
#' @returns a method to build a `cgeneric` should return
#' a named list of `cgeneric` class that contains a
#'  named list `f` that contains (at least):
#'  * `model` a character always equal to `cgeneric`,
#'  * `n` an integer greater than 0, and
#'  * `cgeneric` as a named list that contains the
#'  data needed to define the model. Each element on
#'  ...$f$cgeneric is also a named list containing
#'  `ints`, `doubles`, `characters`, `matrices`
#'  and `smatrices`.
#'  * (possible) `extraconstr` as a named list with: `A` as a
#'   `n` times `k` matrix and `e` as a length `k` vector.
#'
#'  The `cgeneric_shlib_path` function returns a `character`
#'  with the path to the shared lib.
#' @export
#' @examples
#' cg <- cgeneric("iid", n = 10, param = c(1, 0.05))
#' cg
cgeneric <- function(model, ...) {
  UseMethod("cgeneric")
}

#' @describeIn cgeneric-class
#' Call a function named as `cgeneric_[model]` to build a `cgeneric`.
#' @export
cgeneric.character <- function(
    model,
    ...) {

  dotArgs <- list(...)

  ## some "convention" here:
  if(length(grep("cgeneric_", model))==0) {
    model <- paste0("cgeneric_", model)
  }
  if(any(model %in%
         paste0("cgeneric_",
                c("iid", "generic0", "shlib")))) {
    if(!is.null(dotArgs$debug) && dotArgs$debug) {
      cat("call", model, "\n")
    }
    return(do.call(
      what = model,
      args = dotArgs
      )
    )
  } else {

    out <- try(do.call(
      what = model,
      args = dotArgs
    ), silent = TRUE)

    if((inherits(out, "cgeneric")) |
       (inherits(out, "inla.cgeneric"))) {
      return(out)
    } else {

      fn <- findGetFunction(
        fName = model
      )
      if(is.function(fn)) {
        return(cgeneric(model = fn, ...))
      }
    }
  }
}

#' @describeIn cgeneric-class
#' The `cgeneric` method for function.
#' @export
cgeneric.function <- function(
    model,
    ...) {
  out <- do.call(
    what = model,
    args = list(...)
    )
  return(out)
}

#' @describeIn cgeneric-class
#' Check, append `cgeneric` class, returns model unchanged.
#' @export
cgeneric.cgeneric <- function(model, ...) {
  return(model)
}

#' @describeIn cgeneric
#' Check and converts a regular `inla.cgeneric` object to `cgeneric`.
#' @export
cgeneric.inla.cgeneric <- function(model, ...) {
  stopifnot(!c("f") %in% names(model))
  stopifnot(!all(c("model", "n", "cgeneric") %in% names(model$f)))
  stopifnot(!c("data") %in% names(model$f$cgeneric))
  stopifnot(!all(c("ints", "characters") %in% names(model$f$cgeneric)))
  stopifnot(!all(c("n") %in% names(model$f$cgeneric$data$ints)))
  stopifnot(!all(c("shlib") %in% names(model$f$cgeneric$data$characters)))
  class(model) <- c("cgeneric", class(model))
  return(model)
}


#' @describeIn cgeneric
#' Build a `cgeneric` from a list of arguments.
#' @export
cgenericBuilder <- function(
    ...) {

  ## do what INLA::inla.cgeneric.define() does

  dotArgs <- list(...)
  nargs <- names(dotArgs)
  if(any(nargs == ""))
    stop("Please name the arguments!")
  if(!any(nargs == "n"))
    stop("Please provid 'n'!")
  if(!any(nargs == "debug"))
    dotArgs$debug <- FALSE
  if(!any(nargs == "shlib"))
    stop("Please provid 'shlib'!")
  if(!any(nargs == "model"))
    stop("Please provid 'model'!")
  stopifnot(dotArgs$n>=1)
  ind <- pmatch(c("n", "debug"),
                names(dotArgs))
  dotArgs <- c(list(
    n = as.integer(dotArgs$n),
    debug = dotArgs$debug),
    dotArgs[setdiff(1:length(dotArgs),ind)]
  )
  n <- dotArgs$n
  dotArgs$debug <- as.integer(dotArgs$debug)
  dotArgs$model <- as.character(dotArgs$model)
  dotArgs$shlib <- as.character(dotArgs$shlib)

  nM <- length(iM <- which(
    sapply(dotArgs, function(x)
      is(x, "Matrix"))))
  nm <- length(im <- which(
    sapply(dotArgs, is.matrix)))
  ni <- length(ii <- setdiff(
    which(sapply(dotArgs, is.integer)), im))
  nd <- length(id <- setdiff(
    which(sapply(dotArgs, is.double)), im))
  nc <- length(ic <- which(
    sapply(dotArgs, is.character)))
  stopifnot(ni>1)
  stopifnot(ni>1)

  if(dotArgs$debug) {
    cat("The cgeneric model data contains:\n",
        ni, "ints",
        nc, "characters",
        nd, "doubles",
        nm, "matrices, and",
        nM, "smatrices\n")
  }

  cmodel <- dotArgs[c("model", "shlib", "n", "debug")]
  cmodel$data <- vector("list", 5L)
  names(cmodel$data) <- c(
    "ints", "doubles", "characters",
    "matrices", "smatrices")
  cmodel$data$ints <- dotArgs[ii]
  if(nd>0) cmodel$data$doubles <- dotArgs[id]
  cmodel$data$characters <- dotArgs[ic]
  if(nm>0) {
    cmodel$data$matrices <- dotArgs[im]
    for(i in 1:nm) {
      cmodel$data$matrices[[i]] <-
        c(dim(cmodel$data$matrices[[i]]),
          t(cmodel$data$matrices[[i]])) ## columnwise...
    }
  }
  if(nM>0) {
	  cmodel$data$smatrices <- dotArgs[iM]
	  for(i in 1:nM) {
	    smi <- upperPadding(cmodel$data$smatrices[[i]])
	    cmodel$data$smatrices[[i]] <- c(
	      dim(smi), length(smi@x),
	      smi@i, smi@j, smi@x
	    )
	  }
  }
  class(cmodel) <- c("inla.cgeneric.f", ## this is needed in INLA::f() from August 2025
                     "inla.cgeneric") ##  this is needed for older INLA

  cmodel_wrapper <- structure(
    list(
      f = list(
        model = "cgeneric",
        n = n,
        cgeneric = cmodel
      )
    ),
    class = c("cgeneric", "inla.cgeneric")
  )
  if(is.null(dotArgs$mapper)) {
    mapper <- list(n = n)
    class(mapper) <- c("bm_index", "bru_mapper")
    cmodel_wrapper$mapper <- mapper
  } else {
    cmodel_wrapper$mapper <- dotArgs$mapper
  }
  return(cmodel_wrapper)
}

#' @describeIn cgeneric
#'  A default mapper for a cgeneric/rgeneric model
mapper1 <- function(model) {
  vs <- "2.13.0.9005"
  inlabruCheck <- packageCheck(
    name = "inlabru",
    minimum_version = vs) >= vs
  if(is.null(model$mapper) || is.na(inlabruCheck)) {
    mapper <- list(n = model$n)
    class(mapper) <- c("bm_index", "bru_mapper")
  } else {
    bgm <- findGetFunction(
      fName = "bru_get_mapper",
      package = "inlabru")
    mapper <- bgm(model = model)
  }
  return(mapper)
}

#' @describeIn cgeneric
#' Make the lib path for the shared lib of a package.
#' @param package character giving the name of the package
#' that contains the `cgeneric` model.
#' @param useINLAprecomp logical, indicating if it is to use
#' the shared object previously copied and compiled by INLA.
#' @param debug integer, used as verbose in debug.
#' @export
cgeneric_shlib_path <- function(
    package,
    useINLAprecomp,
    debug) {

  if(missing(package) || is.null(package)) {
    stop("please provide package!")
  }
  if(missing(debug)) {
    debug <- FALSE
  }
  if(missing(useINLAprecomp)) {
    useINLAprecomp <- TRUE
  } else {
      if(is.null(useINLAprecomp)) {
          useINLAprecomp <- TRUE
      }
  }

  nbit <- as.integer(8 * (.Machine$sizeof.pointer))
  if(useINLAprecomp) {
    shlib <- paste0(
      find.package("INLA"), "/bin/", apINLAbin(),
      "external/", package, "/lib", package, ".so")
    if(debug) {
      cat("INLA compiled shared lib at:\n", shlib, "\n")
    }
  } else {
    shlib <- paste0(find.package(package = package),
                    "/libs/")
    if (Sys.info()["sysname"] == "Windows") {
      shlib <- file.path(
        shlib,
        paste0("x", nbit, "/", package, ".dll"))
    } else {
      shlib <- file.path(shlib,
                         paste0(package, ".so"))
    }
    if(debug) {
      cat(package, "compiled shared lib at:\n",
          shlib, "\n")
    }
  }
  return(normalizePath(shlib))
}
#' @describeIn cgeneric
#' Print the cgeneric object
#' @param x a cgeneric object
#' @export
print.cgeneric <- function(x, ...) {
  cat("cgeneric: ", x$f$cgeneric$model, ", n = ",
      x$f$cgeneric$n, ", shlib:\n", sep = "")
  cat(x$f$cgeneric$data$characters$shlib, "\n")
  d0 <- c(2L, 0L, 2L, 0, 0)
  dn <- sapply(x$f$cgeneric$data, length)
  dn0 <- dn-d0
  ii <- which(dn0>0)
  if(length(ii)>0) {
    nd <- names(dn)
    for(i in ii){
      d <- x$f$cgeneric$data[[i]]
      ndi <- names(d)
      jj <- (d0[i]+1):dn[i]
      cat(nd[i], " (", dn[i], ") : ", sep = "")
      if(i %in% c(1,2,3)) {
        cat(paste(ndi[jj], "(", sapply(d[jj], length),
                  ") ", sep = ""), "\n")
      } else {
        if(i==4) {
          dj <- sapply(d[jj], function(x) x[1:2])
          cat(paste(ndi[jj], "(", dj[1,], ",", dj[2, ],
                    ") ", sep = ""), "\n")
        }
        if(i==5) {
          dj <- sapply(d[jj], function(x) x[1:3])
          cat(paste(ndi[jj], "(", dj[1,], ",",
                    dj[2, ], ",", dj[3,],
                    ") ", sep = ""), "\n")
        }
      }
    }
  }
}
#' @describeIn cgeneric
#' A summary for a cgeneric object
#' @param object a cgeneric object
#' @export
summary.cgeneric <- function(object, ...) {
  g <- cgeneric_get(object, "graph")
  cat("n = ", object$f$cgeneric$n, ", graph with",
      length(g@x), "non-zeros\n", sep = "")
}
#' @describeIn cgeneric
#' A plot for a cgeneric object
#' @param y not used
#' @export
plot.cgeneric <- function(x, y, ...) {
  g <- cgeneric_get(x, "graph")
  image(g)
}

#' To store in i,j,x sparse matrix format
#' @param A matrix or Matrix
#' @param unique logical (default is TRUE) to
#' ensure that the internal representation
#' is unique and there are no duplicated entries.
#' (Do not change this unless you know what you are doing.)
#' @param na.rm logical (default is FALSE) indicating
#' if it is to replace ‘NA’'s in the matrix with zeros.
#' @param zeros.rm logical (default is FALSE)
#' indicating if it is to remove zeros in the
#' matrix. Applied after `na.rm`.
#' @note
#'  This is based in INLA::inla.as.sparse(),
#' but allow all combinations of 'na.rm' and 'zeros.rm'.
#' @export
Sparse <- function(A,
                   unique = TRUE,
                   na.rm = FALSE,
                   zeros.rm = FALSE) {
  if (!inherits(A, "Matrix")) {
    A <- as(A, "Matrix")
  }
  if (unique) {
    A <- as(as(as(as(A, "dMatrix"),
                  "generalMatrix"),
               "CsparseMatrix"),
            "TsparseMatrix")
  } else {
    if (!inherits(A, "dgTMatrix")) {
      A <- as(as(as(A, "dMatrix"),
                 "generalMatrix"),
              "TsparseMatrix")
    }
  }
  ## different logic than 'INLA::inla.as.sparse()'
  if (na.rm) {
    x.na <- is.na(A@x)
    if (any(x.na)) {
      idx.ok1 <- which(!x.na)
      A@x <- A@x[idx.ok1]
      A@i <- A@i[idx.ok1]
      A@j <- A@j[idx.ok1]
    }
  }
  if (zeros.rm) {
    x.zero <- (A@x==0.0) ## as in the original
    if (any(x.zero)) {
      idx.ok2 <- which(!x.zero)
      A@x <- A@x[idx.ok2]
      A@i <- A@i[idx.ok2]
      A@j <- A@j[idx.ok2]
    }
  }
  return(A)
}
#' Padding (a list of) sparse matrices.
#' @param M 'Matrix' (or a list of them).
#' @param relative logical. If 'M" is a list,
#' it indicates if it is to be returned a relative index
#' and the value for each matrix. See details.
#' @param ... additional arguments passed to [Sparse].
#' @returns If a unique matrix is given, return the
#' upper triangle considering the 'T' representation
#' in the `dgTMatrix`, from the `Matrix` package.
#' If a list of matrices is given,
#' return a list of two elements: 'graph' and 'xx'.
#' The 'graph' is the union of the graph from each matrix.
#' If relative=FALSE, 'xx' is a matrix with number of column equals
#' the the number of matrices imputed.
#' If relative=TRUE, it is a list of length equal the number
#' of matrices imputed. See details.
#' @details
#' This is useful to prepare a matrix, or a list of,
#' sparse matrices for use in some 'cgeneric' code.
#'
#' Define a graph of the union of the supplied matrices
#' and return the row ordered diagonal plus upper triangle
#' after padding with zeroes each one so that
#' all the returned matrices have the same pattern.
#'
#' If relative=FALSE, each columns of 'xx' is the
#' elements of the corresponding matrix after being padded
#' to fill the pattern of the union graph.
#' If relative=TRUE, each element of 'xx' would be a list
#' with a relative index, 'r', for each non-zero elements of
#' each matrix is returned relative to the union graph,
#' the non-lower elements, 'x', of the corresponding matrix,
#' and a vector, 'o', with the number of non-zero elements
#' for each line of each resulting matrix.
#' @export
#' @examples
#' A <- sparseMatrix(
#'   i = c(1, 1, 2, 3, 3, 5),
#'   j = c(2, 5, 3, 4, 5, 5),
#'   x = -c(0:3,NA,1), symmetric = TRUE)
#' A
#' upperPadding(A)
#' upperPadding(A, na.rm = TRUE)
#' upperPadding(A, zeros.rm = TRUE)
#' upperPadding(A, na.rm = TRUE, zeros.rm = TRUE)
#' B <- Diagonal(nrow(A), -colSums(A, na.rm = TRUE))
#' B
#' upperPadding(list(a = A, b = B), na.rm = TRUE, zeros.rm = TRUE)
#' upperPadding(list(a = A, b = B), relative = TRUE)
upperPadding <-
  function(M, relative = FALSE, ...) {
    .uof <- function(m) {
      return(intersect(order(m@i), which(m@j >= m@i)))
    }
    if (is(M, "list")) {
      M <- lapply(M, Sparse,  ...)
      graph <- Sparse(Reduce(
        "+",
        lapply(M, function(m) {
          m@x <- m@x * 0.0 + 1.0
          return(m)
        })
      ))
      uo <- .uof(graph)
      xx <- sapply(M, function(m) {
        m <- graph * 0 + m
        return(m@x[uo])
      })
      graph@i <- graph@i[uo]
      graph@j <- graph@j[uo]
      graph@x <- rep(1, length(uo))
      if (relative) {
        xx <- apply(xx, 2, function(x) {
          r <- which(x != 0)
          return(list(
            r = r, x = x[r]
          ))
        })
      }
      return(list(graph = graph, xx = xx))
    }
    if(is(M, "Matrix")|is(M,"matrix")) {
      M <- Sparse(M, ...)
      uo <- .uof(M)
      M@i <- M@i[uo]
      M@j <- M@j[uo]
      M@x <- M@x[uo]
      return(M)
    }
    return(M)
}

#' Define the is.zero method
#' @param x an R object
#' @param tol numeric to be used as (absolute) tolerance.
#' if missing (default) it will consider `x==0`.
#' @return logical
#' @export
is.zero <- function(x, tol) {
  UseMethod("is.zero")
}
#' @describeIn is.zero
#' The is.zero.default definition
#' @export
is.zero.default <- function(x, tol) {
  if(missing(tol)) {
    return(x==0)
  }
  a <- abs(as.numeric(c(x)))
  return(a < tol)
}
#' @describeIn is.zero
#' The is.zero.matrix definition
#' @export
is.zero.matrix <- function(x, tol) {
  stopifnot(inherits(x, "matrix"))
  return(matrix(is.zero.default(x, tol),
                nrow(x), ncol(x)))
}
#' @describeIn is.zero
#' The is.zero.Matrix definition
#' @export
is.zero.Matrix <- function(x, tol) {
  stopifnot(inherits(x, "Matrix"))
  is.zero(as.matrix(x), tol)
}

#' Define the method to extract the precision from an inla output object.
#' @param model an inla output
#' @param ... used to pass the 'prior' argument,
#' as logical (default is TRUE) to indicate if it is
#' to retrieve the prior or the posterior precision.
#' @details
#' extract the joint prior precision for the
#' latent field at the mode of the hyperparameters
#'
#' @export
inlaQ <- function(model, ...) {
  if(is.null(model$misc$config$config)) {
    warning("inla.rerun() with config = TRUE in control.compute.")
    model$.args$control.compute$config <- TRUE
    model <- do.call("inla", args = model$.args)
  }
  prior <- list(...)$prior
  if(is.null(prior))
    prior <- TRUE
  if(prior) {
    Qu <- Sparse(
      model$misc$config$config[[1]]$Qprior
    )
  } else {
    Qu <- Sparse(
      model$misc$config$config[[1]]$Q
    )
  }
  Q <- Matrix::sparseMatrix(
      i = Qu@i + 1L,
      j = Qu@j + 1L,
      x = Qu@x,
      symmetric = TRUE,
      repr = "T"
    )
  return(Sparse(Q))
}

#' To check package version and load
#' @param name character with the name of the package
#' @param minimum_version character with the minimum required version
#' @param quietly logical indicating if messages shall be printed
#' @note
#' Original in inlabru package function check_package_version_and_load
#' @export
packageCheck <- function(name, minimum_version, quietly = FALSE) {
  version <- tryCatch(utils::packageVersion(name),
                      error = function(e) NA_character_
  )
  if (is.na(version)) {
    if (!quietly) {
      message(paste0("Package '", name, "' is not installed."))
    }
    return(NA_character_)
  }
  if (version < minimum_version) {
    if (!quietly) {
      message(paste0(
        "Installed '", name, "' version is ", version, " but ",
        "version >= ", minimum_version, " is required."
      ))
    }
    return(NA_character_)
  }
  if (!requireNamespace(name, quietly = TRUE)) {
    if (!quietly) {
      message("Package '", name, "' not loaded safely.")
    }
    return(NA_character_)
  }
  return(version)
}
#' Search a function and retrieve it.
#' @param fName character with the name of the function
#' @param package character with the package name
#' @param debug logical indicating if it is to print
#' intermediate progress finding
#' @returns function.
#' The (first) package name where it was found
#' is returned as an attribute named "package"
#' @details
#' if 'missing(package)' it will search on the loaded
#' packages, first in the exported functions, and then
#' among the non-exported ones.
#' NOTE: 'package' can include any installed package,
#' see [installed.packages()]
#' @export
findGetFunction <- function(fName, package, debug = FALSE) {

  if(missing(package)||is.null(package)) {
    pkgs <- .packages()
  } else {
    pkgs <- package
  }

  pkgFound <- NULL
  ## first try exported
  for(i in 1:length(pkgs)) {
    afn <- paste0(pkgs[i], "::", fName)
    efn <- try(eval(str2lang(afn)), silent = TRUE)
    if(is.function(efn)) {
      pkgFound <- pkgs[i]
      if(debug) {
        cat("Found", fName, "in", pkgFound, "!\n")
      }
      break
    }
  }

  if(is.null(pkgFound)) {
    ## try non-exported
    for(i in 1:length(pkgs)) {
      afn <- paste0(pkgs[i], ":::", fName)
      efn <- try(eval(str2lang(afn)), silent = TRUE)
      if(is.function(efn)) {
        pkgFound <- pkgs[i]
        if(debug) {
          cat(fName, "non-exported from", pkgFound, "!\n")
        }
        break
      }
    }
  }

  if(is.null(pkgFound)) {
    stop("Function not found in the tried packages!")
  }

  attr(efn, "package") <- pkgFound
  return(efn)
}
#' Internal: to append path for INLA binary files.
#' @noRd
apINLAbin <- function() {
  nbit <- paste0(as.integer(8 * (.Machine$sizeof.pointer)),
                 "bit/")
  OS <- .Platform$OS.type
  if(OS=="windows") {
    b <- paste0("windows/", nbit)
  } else {
    if(OS=="unix") {
      if((!is.na(file.info("/Library")$isdir)) &&
         (!is.na(file.info("/Applications")$isdir))) {
        if(Sys.info()[["machine"]] == "arm64") {
          b <- paste0("mac.arm64/", nbit)
        } else {
          b <- paste0("mac/", nbit)
        }
      } else {
        b <- paste0("linux/", nbit)
      }
    } else {
      stop("Unknown OS type!")
    }
  }
  return(b)
}

#' `rgeneric_get` is an internal function used by
#' `graph`, `pred`, `initial`, `mu` or `prior`
#' methods for `rgeneric`.
#' @description
#' The `generic_get` retrieve a model property specified by
#' `cmd` on an `rgeneric` object.
#' The functions listed below are for each `cmd` case.
#' @param model a `rgeneric` object.
#' @param cmd an string to specify which model element to get
#' @param theta numeric vector with the model parameters.
#' If missing, the [initial()] will be used.
#' @param ... additional arguments passed on to methods.
#' E.g.: `optimize = FALSE` return the graph and precision
#' as a sparse matrix whereas `optimize = TRUE` retur the
#' graph as arow/col indexes and the precision as a numeric
#' vector with its elements.
#' @return depends on `cmd`
rgeneric_get <- function(model,
                         cmd = c("graph", "Q", "initial", "mu", "log_prior"),
                         theta,
                         ...) {

  ## internal function to
  ## re-implement INLA::inla.rgeneric.q
  ## to be accessed by rgeneric INLAtools methods

  ret <- NULL
  cmd[cmd == "log_prior"] <- "log.prior"
  cmd <- unique(cmd)
  cmds <- c("graph", "Q", "initial", "mu", "log.prior")
  cmd <- match.arg(cmd,
                   cmds,
                   several.ok = TRUE)
  stopifnot(length(cmd)>0)

  optimize <- list(...)$optimize
  if(is.null(optimize))
    optimize <- FALSE

  func <- model$f$rgeneric$definition

  if(length(cmd) == 1) {
    if(cmd == "graph") {
      return(try(func("graph"), silent = TRUE))
    }
    if(cmd == "mu") {
      return(try(func("mu"), silent = TRUE))
    }
  }

  initheta <- try(func("initial"), silent = TRUE)

  if(inherits(initheta, "try-error")) {
    stop('Error trying to get "initial"!')
  }
  if((length(cmd)==1) && (cmd=="initial")) {
    return(initheta)
  }
  theta.size <- length(initheta)

  needtheta <- any(cmd %in% c("Q", "log.prior"))
  if(needtheta & (theta.size>0)) {
    if(missing(theta) || is.null(theta)) {
      warning('missing "theta", using "initial"!')
      theta <- initheta
    }
    theta <- as.double(theta)
    ntheta <- floor(length(theta)/length(initheta))

    ## if more than one theta is given
    if((length(cmd)==1) && (cmd=="log.prior")) {
      if(ntheta==1) {
        return(func("log.prior", theta = theta))
      } else {
        theta <- matrix(theta, nrow = theta.size)
        return(sapply(1:ncol(theta), function(j) {
          func("log.prior", theta = theta[, j])
        }))
      }
    }
  } else {
    theta <- NULL
    ntheta <- 0L
  }

  if(length(cmd) == 1) {
    ret <- try(func(cmd = cmd, theta = theta),
               silent = TRUE)
    if(inherits(ret, "try-error")) {
      stop('Error trying to get "', cmd, '"!')
    }
    if(cmd %in% c("graph", "Q"))
      ret <- Sparse(ret)
    if(optimize) {
      if(cmd == "graph")
        ret <- list(i = as.integer(ret@i),
                    j = as.integer(ret@j))
      if(cmd == "Q")
        ret <- ret@x
    }
  } else {
    names(cmd) <- cmd
    ret <- lapply(cmd, function(x) {
      try(func(cmd = x, theta = theta),
          silent = TRUE)
    })
    if(optimize) {
      if(any(cmd == "graph"))
        if(!inherits(ret$graph, "try-error"))
          ret$graph <- list(
            i = ret$graph@i,
            j = ret$graph@j)
      if(any(cmd == "Q"))
        if(!inherits(ret$Q, "try-error"))
          ret$Q <- ret$Q@x
    }
  }

  return(ret)

}
#' @describeIn rgeneric_get
#' Retrive the initial parameter(s) of an `rgeneric` model.
#' @export
initial.rgeneric <- function(model) {
  rgeneric_get(model, "initial")
}
#' @describeIn rgeneric_get
#' Evaluate the mean for an `rgeneric` model.
#' @export
mu.rgeneric <- function(model, theta) {
  rgeneric_get(model, "mu", theta = theta)
}
#' @describeIn rgeneric_get
#' Retrieve the graph of an `rgeneric` object
#' @param optimize logical indicating if it is to be
#' returned only the elements and not as a sparse matrix.
#' @export
graph.rgeneric <- function(model, optimize) {
  if(missing(optimize)) {
    optimize <- FALSE
  }
  return(rgeneric_get(
    model, "graph",
    optimize = optimize))
}
#' @describeIn rgeneric_get
#' Retrieve the precision of an `rgeneric` object
#' @export
prec.rgeneric <- function(model, theta, optimize) {
  if(missing(theta)) {
    warning('missing "theta", using "initial"!')
    theta <- initial(model)
  }
  if(missing(optimize)) {
    optimize <- FALSE
  }
  rgeneric_get(model,
               cmd = "Q",
               theta = theta,
               optimize = optimize)
}
#' @describeIn rgeneric_get
#' Evaluate the prior for an `rgeneric` model
#' @return numeric scalar (if numeric vector is provided
#' for theta) or vector (if numeric matrix is provided
#' for theta).
#' @export
#' @example demo/prior.R
prior.rgeneric <- function(model, theta) {
  return(rgeneric_get(model = model,
                      cmd = "log_prior",
                      theta = theta))
}

#' Internal functions to check PC-prior parameters.
#' @name pc-utils
NULL
#> NULL

#' @describeIn pc-utils
#' Check the PC-prior arguments.
#' @param npars number of parameters.
#' @param reference numeric vector to set the reference
#' for each parameter for its PC-prior.
#' @param probability numeric vector with to
#' set the probability statement of the PC prior for each
#' parameter. For `sigma` probability statement is
#' P(sigma > `reference`) = p whereas for `range` it is
#' P(range < `reference`). If  NA, 0 or 1, the corresponding
#' `reference` will be used as fixed. If missing, all the
#' parameters considered as known (fixed) and equal the
#' corresponding reference value.
#' @export
pcParamCheck <- function(npars,
                         reference,
                         probability) {
  if(missing(reference)) {
    reference <- rep(1, npars)
  }
  if(length(reference)<1) {
    reference <-
      rep(reference, npars)[1:npars]
  }
  if(missing(probability)) {
    probability <- rep(0, npars)
  }
  if(length(probability)<1) {
    probability <-
      rep(probability, npars)[1:npars]
  }
  stopifnot(length(reference) == npars)
  stopifnot(length(probability) == npars)
  stopifnot(all(reference>0))
  probability[is.na(probability)] <- 0
  stopifnot(all(probability>=0.0))
  stopifnot(all(probability<=1.0))
  fixed <- is.zero(probability) |
    is.zero(1-probability)
  return(list(
    reference = reference,
    probability = probability,
    fixed = fixed
  ))
}
#' @describeIn pc-utils
#' Penalized Complexity (PC) prior for the log of the practical range.
#' @param lrange numeric with the log of the (practical) range
#' @param lam numeric with the prior parameter
#' @param d integer to specify the domain dimention
#' @param log logical indicating if the density
#' is to be returned in the log scale
#' @export
pclrange <- function(lrange, lam, d = 2, log = FALSE) {
  dh <- 0.5 * d
  out <- log(lam * dh) -dh * lrange - lam * exp(-dh * lrange)
  if(log)
    return(out)
  return(exp(out))
}
#' @describeIn pc-utils
#' Penalized Complexity (PC) prior for the practical range.
#' @param range numeric with the of the (practical) range.
#' @export
#' @examples
#' # P(range < 2.0) = 0.1
#'  lam <- -log(0.1) * 2.0
#'  plot(function(x) pcrange(x, lam), 1/100, 10, n = 100)
pcrange <- function(range, lam, d = 2, log = FALSE) {
  pclrange(log(range), lam, d = d, log = log)/range
}

#' Build a `cgeneric` object for a `generic0` model.
#' @description
#' Build data needed to implement a model whose
#' precision has a conditional precision parameter.
#' This uses the C interface in the 'INLA' package,
#' that can be used as a linear predictor
#' model component with an 'f' term.
#' @param R the structure matrix for the model definition.
#' @param param length two vector with the parameters
#' `a` and `p` for the PC-prior distribution defined from
#'   \deqn{P(\sigma > a) = p}
#' where \eqn{\sigma} can be interpreted as marginal standard
#' deviation of the process if scale = TRUE. See details.
#' @param constr logical indicating if it is to add a
#' sum-to-zero constraint. Default is TRUE.
#' @param scale logical indicating if it is to scale
#' the model. See detais.
#' @param ... arguments (debug,useINLAprecomp,shlib)
#' passed on to `cgeneric`.
#' @details
#' The precision matrix is defined as
#'  \deqn{Q = \tau R}
#' where the structure matrix R is supplied by the user
#' and \eqn{\tau} is the precision parameter.
#' Following Sørbie and Rue (2014), if scale = TRUE
#' the model is scaled so that
#'  \deqn{Q = \tau s R}
#'  where \eqn{s} is the geometric mean of the diagonal
#'  elements of the generalized inverse of \eqn{R}.
#' \deqn{s = \exp{\sum_i \log((R^{-})_{ii})/n}}
#' If the model is scaled, the geometric mean of the
#' marginal variances, the diagonal of \eqn{Q^{-1}}, is one.
#' Therefore, when the model is scaled,
#' \eqn{\tau} is the marginal precision,
#' otherwise \eqn{\tau} is the conditional precision.
#' @references
#' Sigrunn Holbek Sørbye and Håvard Rue (2014).
#' Scaling intrinsic Gaussian Markov random field priors in
#' spatial modelling. Spatial Statistics, vol. 8, p. 39-51.
#' @return a `cgeneric` object, see [cgeneric-class()].
#' @export
cgeneric_generic0 <-
  function(R,
           param,
           constr = TRUE,
           scale = TRUE,
           ...) {

    stopifnot(param[1]>0)
    if(is.na(param[2])) {
      param[2] <- 0.0
    }
    stopifnot(param[2]>=0)
    stopifnot(param[2]<=1)

    dotArgs <- list(...)
    if(is.null(dotArgs$debug)) {
      dotArgs$debug <- FALSE
    }

    stopifnot(nrow(R)==ncol(R))
    stopifnot(nrow(R)>0)
    n <- as.integer(nrow(R))
    R <- upperPadding(
      R + sparseMatrix(i = 1:n, j = 1:n, x = rep(0.0, n)))
    if(dotArgs$debug) {
      print(str(R))
    }

    if(scale | constr) {
      ## This work with dense matrices
      ## For big matrices, consider INLA:::inla.scale.model()
      Rd <- as.matrix(R)
      R.svd <- svd(Rd + t(Rd) - diag(diag(Rd)))
      R.svd$ip <- which(R.svd$d>sqrt(.Machine$double.eps))
      if(dotArgs$debug) {
        print(str(list(R.svd=R.svd)))
      }
      stopifnot(length(R.svd$ip)>0)
    }

    if(scale) {
      R.gi <- R.svd$v[, R.svd$ip, drop = FALSE] %*% (
          (1/R.svd$d[R.svd$ip])*t(R.svd$u[, R.svd$ip, drop=FALSE]))
      R <- R * exp(mean(log(diag(R.gi))))
    }
    if(is.null(dotArgs$useINLAprecomp)) {
      dotArgs$useINLAprecomp <- TRUE
    }
    INLAvcheck <- packageCheck(
      name = "INLA",
      minimum_version = "25.10.28")
    if(is.na(INLAvcheck) & dotArgs$useINLAprecomp) {
      dotArgs$useINLAprecomp <- FALSE
      warning("INLA version is old. Setting 'useINLAprecomp = FALSE'!")
    }
    dotArgs$shlib <- cgeneric_shlib_path(
      package = "INLAtools",
      useINLAprecomp = dotArgs$useINLAprecomp,
      debug = dotArgs$debug)

    the_model <- do.call(
      what = "cgenericBuilder",
      args = c(list(
        model = "inla_cgeneric_generic0",
        n=as.integer(n),
        param=param,
        Rgraph = R),
        dotArgs)
    )
## setup extraconstr
    Ae <- dotArgs$extraconstr
    if(!is.null(Ae)) {
      stopifnot(!(names(Ae) %in% c("A", "e")))
      stopifnot(ncol(Ae$A)==n)
      stopifnot(nrow(Ae$A)==length(Ae$e))
    }
    if(constr) {
      the_model$f$constr <- FALSE
      jj <- setdiff(1:n, R.svd$ip)
      Ae$A <- rbind(Ae$A,
                    t(R.svd$u[, jj, drop = FALSE]))
      Ae$e <- c(Ae$e, rep(0, length(jj)))
      qrc <- qr(t(Ae$A))
      c.ok <- which(abs(qrc$qraux)>0)
      Ae$A <- Ae$A[qrc$pivot[c.ok], , drop = FALSE]
      Ae$e <- Ae$e[qrc$pivot[c.ok]]
    }
    the_model$f$extraconstr <- Ae

    return(the_model)

  }
#' @describeIn cgeneric_generic0
#' The [cgeneric_iid] uses the [cgeneric_generic0]
#' with the structure matrix as the identity.
#' @param n integer required to specify the model size
#' @export
cgeneric_iid <-
  function(n,
           param,
           constr = FALSE,
           ...) {
    do.call(
      what = "cgeneric_generic0",
      args = list(
        R = Diagonal(n = n, x = rep(1, n)),
        param = param,
        constr = constr,
        scale = FALSE,
        ...)
    )
  }

#' `rgeneric_get` is an internal function used to query
#' `graph`, `Q`, `initial`, `mu` or `prior` from a `rgeneric`.
#' @description
#' The `generic_get` retrieve a model property specified by
#' `cmd` on an `rgeneric` object.
#' The functions listed below are for each `cmd` case.
#' @param model a `rgeneric` object.
#' @param cmd an string to specify which model element to get
#' @param theta numeric vector with the model parameters.
#' If missing, the `initial` will be used.
#' @param ... additional arguments passed on to methods.
#' E.g.: `optimize = FALSE` return the graph and precision
#' as a sparse matrix whereas `optimize = TRUE` retur the
#' graph as arow/col indexes and the precision as a numeric
#' vector with its elements.
#' @return depends on `cmd`
rgeneric_get <- function(model,
                         cmd = c("graph", "Q", "initial", "mu", "log_prior"),
                         theta,
                         ...) {

  ## internal function to
  ## re-implement INLA::inla.rgeneric.q
  ## to be accessed by rgeneric INLAtools methods

  ret <- NULL
  cmd[cmd == "log_prior"] <- "log.prior"
  cmd <- unique(cmd)
  cmds <- c("graph", "Q", "initial", "mu", "log.prior")
  cmd <- match.arg(cmd,
                   cmds,
                   several.ok = TRUE)
  stopifnot(length(cmd)>0)

  optimize <- list(...)$optimize
  if(is.null(optimize))
    optimize <- FALSE

  func <- model$f$rgeneric$definition

  if(length(cmd) == 1) {
    if(cmd == "graph") {
      return(try(func("graph"), silent = TRUE))
    }
    if(cmd == "mu") {
      return(try(func("mu"), silent = TRUE))
    }
  }

  initheta <- try(func("initial"), silent = TRUE)

  if(inherits(initheta, "try-error")) {
    stop('Error trying to get "initial"!')
  }
  if((length(cmd)==1) && (cmd=="initial")) {
    return(initheta)
  }
  theta.size <- length(initheta)

  needtheta <- any(cmd %in% c("Q", "log.prior"))
  if(needtheta & (theta.size>0)) {
    if(missing(theta) || is.null(theta)) {
      warning('missing "theta", using "initial"!')
      theta <- initheta
    }
    theta <- as.double(theta)
    ntheta <- floor(length(theta)/length(initheta))

    ## if more than one theta is given
    if((length(cmd)==1) && (cmd=="log.prior")) {
      if(ntheta==1) {
        return(func("log.prior", theta = theta))
      } else {
        theta <- matrix(theta, nrow = theta.size)
        return(sapply(1:ncol(theta), function(j) {
          func("log.prior", theta = theta[, j])
        }))
      }
    }
  } else {
    theta <- NULL
    ntheta <- 0L
  }

  if(length(cmd) == 1) {
    ret <- try(func(cmd = cmd, theta = theta),
               silent = TRUE)
    if(inherits(ret, "try-error")) {
      stop('Error trying to get "', cmd, '"!')
    }
    if(cmd %in% c("graph", "Q"))
      ret <- Sparse(ret)
    if(optimize) {
      if(cmd == "graph")
        ret <- list(i = as.integer(ret@i),
                    j = as.integer(ret@j))
      if(cmd == "Q")
        ret <- ret@x
    }
  } else {
    names(cmd) <- cmd
    ret <- lapply(cmd, function(x) {
      try(func(cmd = x, theta = theta),
          silent = TRUE)
    })
    if(optimize) {
      if(any(cmd == "graph"))
        if(!inherits(ret$graph, "try-error"))
          ret$graph <- list(
            i = ret$graph@i,
            j = ret$graph@j)
      if(any(cmd == "Q"))
        if(!inherits(ret$Q, "try-error"))
          ret$Q <- ret$Q@x
    }
  }

  return(ret)

}
#' @describeIn rgeneric_get
#' Retrive the initial parameter(s) of an `rgeneric` model.
#' @export
rgeneric_initial <- function(model) {
  rgeneric_get(model, "initial")
}
#' @describeIn rgeneric_get
#' Evaluate the mean for an `rgeneric` model.
#' @export
rgeneric_mu <- function(model, theta) {
  rgeneric_get(model, "mu", theta = theta)
}
#' @describeIn rgeneric_get
#' Retrieve the graph of an `rgeneric` object
#' @param optimize logical indicating if it is to be
#' returned only the elements and not as a sparse matrix.
#' @export
rgeneric_graph <- function(model, optimize) {
  if(missing(optimize)) {
    optimize <- FALSE
  }
  return(rgeneric_get(
    model, "graph",
    optimize = optimize))
}
#' @describeIn rgeneric_get
#' Retrieve the precision of an `rgeneric` object
#' @export
rgeneric_Q <- function(model, theta, optimize) {
  if(missing(optimize)) {
    optimize <- FALSE
  }
  rgeneric_get(model,
               cmd = "Q",
               theta = theta,
               optimize = optimize)
}
#' @describeIn rgeneric_get
#' Evaluate the prior for an `rgeneric` model
#' @return numeric scalar (if numeric vector is provided
#' for theta) or vector (if numeric matrix is provided
#' for theta).
#' @export
#' @example demo/prior.R
rgeneric_prior <- function(model, theta) {
  return(rgeneric_get(model = model,
                      cmd = "log_prior",
                      theta = theta))
}

#' Organize data for the latent GMRF C interface for `INLA`.
#' @rdname cgeneric-class
setClass(
  "cgeneric",
  slots = c(f = "list"),
  validity = function(object) {
    d <- b <- FALSE
    a <- all(c("model", "n", "cgeneric") %in%
               names(object$f))
    if(a) {
      b <- all(c("model", "shlib", "n", "debug", "data")
               %in% names(object$f$cgeneric))
    }
    if(a & b) {
      d <- all(c("ints", "doubles", "characters", "matrices", "smatrices")
               %in% names(object$f$cgeneric$data))
    }
    a & b & d
  }
)
#' Organize data for the latent GMRF R interface for `INLA`.
#' @rdname rgeneric-class
setClass(
  "rgeneric",
  slots = c(f = "list"),
  validity = function(object) {
    all(c("model", "n", "rgeneric") %in%
          names(object$f))
  }
)

#' @rdname rgeneric-class
#' @inherit cgeneric-class description
#' @param model an object used to define the model.
#' See the 'rgeneric' vignette from the INLA package.
#' @param debug logical indicating debug state.
#' @param n integer with the dimension of the model
#' @param ... additional arguments to be used internally
#' for the model, for example, additional data.
#' @inherit cgeneric details
#' @note
#' Recommended for prototyping, whereas
#' `cgeneric` is recommended for production.
#' @returns `rgeneric`/ `inla.rgeneric` object.
#' @export
rgeneric <- function(model,
                     n,
                     debug = FALSE,
                     ...) {
  UseMethod("rgeneric")
}
#' @describeIn rgeneric
#' The rgeneric default method.
#' @export
rgeneric.default <- function(model,
                             n,
                             debug = FALSE,
                             ...) {
    dArgs <- list(...)
    stopifnot(!any(names(dArgs)==""))

    env <- if (length(dArgs) > 0) as.environment(dArgs) else new.env()
    parent.env(env) <- .GlobalEnv
    environment(model) <- env

  ## as INLA::inla.rgeneric.define(..., compile = TRUE, optimize = TRUE)
  rmodel <- structure(
    list(
      f = list(
        model = "rgeneric",
        n = n,
        rgeneric = structure(
          list(
            definition =
              compiler::cmpfun(
                model,
                options = list(optimize = 3L)),
            debug = debug,
            optimize = TRUE
          ),
          # inla.rgeneric is needed to support INLA before August 2025
          class = c("inla.rgeneric.f", "inla.rgeneric")
        )
      )
    ),
    class = c("rgeneric", "inla.rgeneric")
  )

  return(rmodel)
}

#' @describeIn rgeneric Returns the model object unchanged.
#' @export
rgeneric.rgeneric <- function(model, ...) {
  return(model)
}

#' @describeIn rgeneric Check and converts a regular `inla.rgeneric` object to `rgeneric`.
#' @export
rgeneric.inla.rgeneric <- function(model, ...) {
  stopifnot(!c("f") %in% names(model))
  stopifnot(!all(c("model", "n", "rgeneric") %in%
                   names(model$f)))
  stopifnot(!all(c("definition", "debug", "optimize") %in%
                   names(model$f$rgeneric)))
  class(model) <- c("rgeneric", class(model))
  return(model)
}

#' @describeIn rgeneric Print the rgeneric object
#' @param x a rgeneric object
#' @param ... not used
#' @export
print.rgeneric <- function(x, ...) {
  cat("rgeneric: ", x$f$rgeneric$model, ", n = ",
      x$f$rgeneric$n, "\n", sep = "")
}
#' @describeIn rgeneric A summary for a rgeneric object
#' @param object a rgeneric object
#' @param ... not used
#' @export
summary.rgeneric <- function(object, ...) {
  g <- rgeneric_get(object, "graph")
  cat("n = ", object$f$rgeneric$n, ", graph with",
      length(g@x), "non-zeros\n", sep = "")
}
#' @describeIn rgeneric A plot for a rgeneric object
#' @param y not used
#' @export
plot.rgeneric <- function(x, y, ...) {
  g <- rgeneric_get(x, "graph")
  image(g)
}

#' `cgeneric_get` is an internal function used by
#' `graph`, `pred`, `initial`, `mu` or `prior`
#' methods for `cgeneric`.
#' @description
#' The `generic_get` retrieve a model property specified by
#' `cmd` on an `cgeneric` object.
#' The functions listed below are for each `cmd` case.
#' @param model a `cgeneric` object.
#' @param cmd an string to specify which model element to get
#' @param theta numeric vector with the model parameters.
#' If missing, the [initial()] will be used.
#' @param optimize logical. If missing or FALSE,
#' the graph and precision are as a sparse matrix.
#' If TRUE, graph only return the row/col indexes and
#' precision return only the elements as a vector.
#' @useDynLib INLAtools
#' @return depends on `cmd`
#' @seealso check the examples in [cgeneric_generic0()]
cgeneric_get <- function(model,
                         cmd = c("graph", "Q", "initial", "mu", "log_prior"),
                         theta,
                         optimize = TRUE) {

  stopifnot(model$f$n>0)
  n <- model$f$n

  ret <- NULL
  cmd[cmd == "log.prior"] <- "log_prior"
  cmd <- unique(cmd)

  ##  print(c(cmd = cmd))

  cgdata <- model$f$cgeneric$data
  stopifnot(!is.null(cgdata))
  stopifnot(!is.null(cgdata$ints))
  stopifnot(!is.null(cgdata$characters))

  ## from Version 0.0.3.902 (and src/cgeneric_get.c)
  ## split each sparse matrix into list elements:
  ## nr, nc, m, i, j, x
  nsm <- length(cgdata$smatrices)
  if(nsm>0) {
    for(i in 1:nsm) {
      smi <- cgdata$smatrices[[i]]
      mi <- as.integer(smi[3])
      cgdata$smatrices[[i]] <- list(
        nr = as.integer(smi[1]),
        nc = as.integer(smi[2]),
        m = mi,
        i = as.integer(smi[3+1:mi]),
        j = as.integer(smi[3+mi+1:mi]),
        x = smi[3+2*mi+1:mi]
      )
    }
  }

  cmds <- c("graph", "Q", "initial", "mu", "log_prior")
  cmd <- match.arg(cmd,
                   cmds,
                   several.ok = TRUE)
  stopifnot(length(cmd)>0)

  initheta <- try(.Call(
    "inla_cgeneric_element_get",
    "initial",
    NULL,
    as.integer(1),
    cgdata$ints,
    cgdata$doubles,
    cgdata$characters,
    cgdata$matrices,
    cgdata$smatrices,
    PACKAGE = "INLAtools"
  ), silent = TRUE)
  if(inherits(initheta, "try-error")) {
    cat("Problem with the current `shlib`:")
    print(attr(initheta, "condition")$message)
    ## workaround: swap   shlib
    ## Note: cgeneric Kronecker not considered
    sshlib <- strsplit(cgdata$characters$shlib, "/")[[1]]
    lpkg <- utils::tail(sshlib, 2)[1]
    ish <- cgeneric_shlib(
      package = lpkg,
      useINLAprecomp = FALSE)
    psh <- cgeneric_shlib(
      package = lpkg,
      useINLAprecomp = FALSE)
    if(cgdata$characters$shlib == ish) {
      cgdata$characters$shlib <- psh
    } else {
      cgdata$characters$shlib <- ish
    }
    warning(paste("Changed `shlib` to\n",
                  cgdata$characters$shlib))
    initheta <- try(.Call(
      "inla_cgeneric_element_get",
      "initial",
      NULL,
      as.integer(1),
      cgdata$ints,
      cgdata$doubles,
      cgdata$characters,
      cgdata$matrices,
      cgdata$smatrices,
      PACKAGE = "INLAtools"
    ), silent = TRUE)
    if(inherits(initheta, "try-error")) {
      print(attr(initheta, "condition")$message)
      stop('Error trying to get "initial"!')
    }
  }
  if((length(cmd)==1) && (cmd=="initial")) {
    return(initheta)
  }
  theta.size <- length(initheta)

  needtheta <- any(cmd %in% c("Q", "log_prior"))
  if(needtheta & (theta.size>0)) {
    if(missing(theta)) {
      warning('missing "theta", using "initial"!')
      theta <- initheta
    } else {
      if(is.null(theta)) {
        warning('NULL "theta", using "initial"!')
        theta <- initheta
      }
    }
    theta <- as.double(theta)
    stopifnot((length(theta)%%theta.size)==0)
    ntheta <- floor(length(theta)/length(initheta))
  } else {
    theta <- NULL
    ntheta <- 0L
  }

  if(any(cmd %in% c("graph", "Q")) & (!optimize)) {
    ij2Q <- function(ij, x = NULL) {
      if(is.null(x)) {
        x <- rep(1L, length(ij[[1]]))
      }
      Sparse(Matrix::sparseMatrix(
        i = ij[[1]] + 1L,
        j = ij[[2]] + 1L,
        x = x,
        symmetric = TRUE,
        repr = "T",
        dims = c(n, n)
      ))
    }
  }

  if(length(cmd) == 1) {
    ret <- try(.Call(
      "inla_cgeneric_element_get",
      cmd,
      theta,
      as.integer(ntheta),
      cgdata$ints,
      cgdata$doubles,
      cgdata$characters,
      cgdata$matrices,
      cgdata$smatrices,
      PACKAGE = "INLAtools"
    ), silent = TRUE)
    if(inherits(ret, "try-error")) {
      print(attr(ret, "condition")$message)
      stop('Error trying to get "', cmd, '"!')
    }

    if((cmd %in% c("graph", "Q")) && (!optimize)) {
      if(cmd == "graph") {
        ij <- ret
        ret <- rep(1, length(ij[[1]]))
      } else {
        ij <- .Call(
          "inla_cgeneric_element_get",
          "graph",
          NULL,
          as.integer(ntheta),
          cgdata$ints,
          cgdata$doubles,
          cgdata$characters,
          cgdata$matrices,
          cgdata$smatrices,
          PACKAGE = "INLAtools"
        )
      }
      ret <- ij2Q(ij, ret)
    }
    return(ret)
  }

  names(cmd) <- cmd
  ret <-
    lapply(
      cmd, function(x) {
        .Call(
          "inla_cgeneric_element_get",
          x,
          theta,
          as.integer(ntheta),
          cgdata$ints,
          cgdata$doubles,
          cgdata$characters,
          cgdata$matrices,
          cgdata$smatrices,
          PACKAGE = "INLAtools"
        )
      }
    )
  if(optimize) {
    return(ret)
  }

  if(any(cmd == "Q")) {
    if(any(cmd == "graph")) {
      ret$Q <- ret$graph <- ij2Q(ret$graph, ret$Q)
      ret$graph@x <- rep(1L, length(graph@x))
    } else {
      ret$Q <- ij2Q(
        .Call(
          "inla_cgeneric_element_get",
          "graph",
          theta,
          as.integer(ntheta),
          cgdata$ints,
          cgdata$doubles,
          cgdata$characters,
          cgdata$matrices,
          cgdata$smatrices,
          PACKAGE = "INLAtools"
        ),
        x = ret$Q
      )
    }
  }

  return(ret)

}
#' @describeIn cgeneric_get
#' Retrive the initial parameter(s) of an `cgeneric` model.
#' @export
initial.cgeneric <- function(model) {
  cgeneric_get(model, "initial")
}
#' @describeIn cgeneric_get
#' Evaluate the mean for an `cgeneric` model.
#' @export
mu.cgeneric <- function(model, theta) {
  cgeneric_get(model, "mu", theta = theta)
}
#' @describeIn cgeneric_get
#' Retrieve the graph of an `cgeneric` object
#' @param optimize logical indicating if it is to be
#' returned only the elements and not as a sparse matrix.
#' @export
graph.cgeneric <- function(model, optimize) {
  if(missing(optimize)) {
    optimize <- FALSE
  }
  return(cgeneric_get(
    model, "graph",
    optimize = optimize))
}
#' @describeIn cgeneric_get
#' Retrieve the precision of an `cgeneric` object
#' @export
prec.cgeneric <- function(model, theta, optimize) {
  if(missing(theta)) {
    warning('missing "theta", using "initial"!')
    theta <- initial(model)
  }
  if(missing(optimize)) {
    optimize <- FALSE
  }
  cgeneric_get(model,
               cmd = "Q",
               theta = theta,
               optimize = optimize)
}
#' @describeIn cgeneric_get
#' Evaluate the prior for an `cgeneric` model
#' @return numeric scalar (if numeric vector is provided
#' for theta) or vector (if numeric matrix is provided
#' for theta).
#' @export
#' @example demo/prior.R
prior.cgeneric <- function(model, theta) {
  return(cgeneric_get(model = model,
                      cmd = "log_prior",
                      theta = theta))
}

#' Internal to make NAMESPACE
#' @noRd
#' @importFrom methods as is
#' @importFrom stats vcov
#' @importFrom utils str
#' @importFrom graphics image
#' @importFrom Matrix diag Diagonal as.matrix sparseMatrix
#' @useDynLib INLAtools
NULL

#' `cgeneric_get` is an internal function used to query
#' `graph`, `Q`, `initial`, `mu` or `log_prior` from a
#' `cgeneric` model.
#' @description
#' The `generic_get` retrieve a model property specified by
#' `cmd` on an `cgeneric` object.
#' The functions listed below are for each `cmd` case.
#' @param model a `cgeneric` object.
#' @param cmd an string to specify which model element to get
#' @param theta numeric vector with the model parameters.
#' If missing, the `initial` will be used.
#' @param optimize logical. If missing or FALSE,
#' the graph and precision are as a sparse matrix.
#' If TRUE, graph only return the row/col indexes and
#' precision return only the elements as a vector.
#' @return depends on `cmd`
#' @seealso check the examples in [cgeneric_generic0()]
cgeneric_get <- function(model,
                         cmd = c("graph", "Q", "initial", "mu", "log_prior"),
                         theta,
                         optimize = TRUE) {

  stopifnot(model$f$n>0)
  n <- model$f$n

  ret <- NULL
  cmd[cmd == "log.prior"] <- "log_prior"
  cmd <- unique(cmd)

  ##  print(c(cmd = cmd))

  cgdata <- model$f$cgeneric$data
  stopifnot(!is.null(cgdata))
  stopifnot(!is.null(cgdata$ints))
  stopifnot(!is.null(cgdata$characters))

  ## from Version 0.0.3.902 (and src/cgeneric_get.c)
  ## split each sparse matrix into list elements:
  ## nr, nc, m, i, j, x
  nsm <- length(cgdata$smatrices)
  if(nsm>0) {
    for(i in 1:nsm) {
      smi <- cgdata$smatrices[[i]]
      mi <- as.integer(smi[3])
      cgdata$smatrices[[i]] <- list(
        nr = as.integer(smi[1]),
        nc = as.integer(smi[2]),
        m = mi,
        i = as.integer(smi[3+1:mi]),
        j = as.integer(smi[3+mi+1:mi]),
        x = smi[3+2*mi+1:mi]
      )
    }
  }

  cmds <- c("graph", "Q", "initial", "mu", "log_prior")
  cmd <- match.arg(cmd,
                   cmds,
                   several.ok = TRUE)
  stopifnot(length(cmd)>0)

  initheta <- try(.Call(
    "inla_cgeneric_element_get",
    "initial",
    NULL,
    as.integer(1),
    cgdata$ints,
    cgdata$doubles,
    cgdata$characters,
    cgdata$matrices,
    cgdata$smatrices,
    PACKAGE = "INLAtools"
  ), silent = TRUE)
  if(inherits(initheta, "try-error")) {
    cat("Problem with the current `shlib`:")
    print(attr(initheta, "condition")$message)
    ## workaround: swap   shlib
    ## Note: cgeneric Kronecker not considered
    sshlib <- strsplit(cgdata$characters$shlib, "/")[[1]]
    lpkg <- utils::tail(sshlib, 2)[1]
    ish <- cgeneric_shlib_path(
      package = lpkg,
      useINLAprecomp = FALSE)
    psh <- cgeneric_shlib_path(
      package = lpkg,
      useINLAprecomp = FALSE)
    if(cgdata$characters$shlib == ish) {
      cgdata$characters$shlib <- psh
    } else {
      cgdata$characters$shlib <- ish
    }
    warning(paste("Changed `shlib` to\n",
                  cgdata$characters$shlib))
    initheta <- try(.Call(
      "inla_cgeneric_element_get",
      "initial",
      NULL,
      as.integer(1),
      cgdata$ints,
      cgdata$doubles,
      cgdata$characters,
      cgdata$matrices,
      cgdata$smatrices,
      PACKAGE = "INLAtools"
    ), silent = TRUE)
    if(inherits(initheta, "try-error")) {
      print(attr(initheta, "condition")$message)
      stop('Error trying to get "initial"!')
    }
  }
  if((length(cmd)==1) && (cmd=="initial")) {
    return(initheta)
  }
  theta.size <- length(initheta)

  needtheta <- any(cmd %in% c("Q", "log_prior"))
  if(needtheta & (theta.size>0)) {
    if(missing(theta)) {
      warning('missing "theta", using "initial"!')
      theta <- initheta
    } else {
      if(is.null(theta)) {
        warning('NULL "theta", using "initial"!')
        theta <- initheta
      }
    }
    theta <- as.double(theta)
    stopifnot((length(theta)%%theta.size)==0)
    ntheta <- floor(length(theta)/length(initheta))
  } else {
    theta <- NULL
    ntheta <- 0L
  }

  if(any(cmd %in% c("graph", "Q")) & (!optimize)) {
    ij2Q <- function(ij, x = NULL) {
      if(is.null(x)) {
        x <- rep(1L, length(ij[[1]]))
      }
      Sparse(Matrix::sparseMatrix(
        i = ij[[1]] + 1L,
        j = ij[[2]] + 1L,
        x = x,
        symmetric = TRUE,
        repr = "T",
        dims = c(n, n)
      ))
    }
  }

  if(length(cmd) == 1) {
    ret <- try(.Call(
      "inla_cgeneric_element_get",
      cmd,
      theta,
      as.integer(ntheta),
      cgdata$ints,
      cgdata$doubles,
      cgdata$characters,
      cgdata$matrices,
      cgdata$smatrices,
      PACKAGE = "INLAtools"
    ), silent = TRUE)
    if(inherits(ret, "try-error")) {
      print(attr(ret, "condition")$message)
      stop('Error trying to get "', cmd, '"!')
    }

    if((cmd %in% c("graph", "Q")) && (!optimize)) {
      if(cmd == "graph") {
        ij <- ret
        ret <- rep(1, length(ij[[1]]))
      } else {
        ij <- .Call(
          "inla_cgeneric_element_get",
          "graph",
          NULL,
          as.integer(ntheta),
          cgdata$ints,
          cgdata$doubles,
          cgdata$characters,
          cgdata$matrices,
          cgdata$smatrices,
          PACKAGE = "INLAtools"
        )
      }
      ret <- ij2Q(ij, ret)
    }
    return(ret)
  }

  names(cmd) <- cmd
  ret <-
    lapply(
      cmd, function(x) {
        .Call(
          "inla_cgeneric_element_get",
          x,
          theta,
          as.integer(ntheta),
          cgdata$ints,
          cgdata$doubles,
          cgdata$characters,
          cgdata$matrices,
          cgdata$smatrices,
          PACKAGE = "INLAtools"
        )
      }
    )
  if(optimize) {
    return(ret)
  }

  if(any(cmd == "Q")) {
    if(any(cmd == "graph")) {
      ret$Q <- ret$graph <- ij2Q(ret$graph, ret$Q)
      ret$graph@x <- rep(1L, length(ret$graph@x))
    } else {
      ret$Q <- ij2Q(
        .Call(
          "inla_cgeneric_element_get",
          "graph",
          theta,
          as.integer(ntheta),
          cgdata$ints,
          cgdata$doubles,
          cgdata$characters,
          cgdata$matrices,
          cgdata$smatrices,
          PACKAGE = "INLAtools"
        ),
        x = ret$Q
      )
    }
  }

  return(ret)

}
#' @describeIn cgeneric_get
#' Retrive the initial parameter(s) of an `cgeneric` model.
#' @export
cgeneric_initial <- function(model) {
  cgeneric_get(model, "initial")
}
#' @describeIn cgeneric_get
#' Evaluate the mean for an `cgeneric` model.
#' @export
cgeneric_mu <- function(model, theta) {
  cgeneric_get(model, "mu", theta = theta)
}
#' @describeIn cgeneric_get
#' Retrieve the graph of an `cgeneric` object
#' @param optimize logical indicating if it is to be
#' returned only the elements and not as a sparse matrix.
#' @export
cgeneric_graph <- function(model, optimize) {
  if(missing(optimize)) {
    optimize <- FALSE
  }
  return(cgeneric_get(
    model, "graph",
    optimize = optimize))
}
#' @describeIn cgeneric_get
#' Retrieve the precision of an `cgeneric` object
#' @export
cgeneric_Q <- function(model, theta, optimize) {
  if(missing(optimize)) {
    optimize <- FALSE
  }
  cgeneric_get(model,
               cmd = "Q",
               theta = theta,
               optimize = optimize)
}
#' @describeIn cgeneric_get
#' Evaluate the prior for an `cgeneric` model
#' @return numeric scalar (if numeric vector is provided
#' for theta) or vector (if numeric matrix is provided
#' for theta).
#' @export
#' @example demo/prior.R
cgeneric_prior <- function(model, theta) {
  return(cgeneric_get(model = model,
                      cmd = "log_prior",
                      theta = theta))
}

#' Kronecker between `cgeneric`|`rgeneric` to implement interaction
#' between GMRF models.
#' @name kronecker
#' @param X `cgeneric` or `rgeneric`
#' @param Y `cgeneric` or `rgeneric`
#' @param FUN see [kronecker()]
#' @param make.dimnames see [kronecker()]
#' @param ... see [kronecker()]
#' @returns A `cgeneric` object if kronecker between two `cgeneric`,
#' otherwise a `rgeneric` object.
NULL
#> NULL

#' @describeIn kronecker
#' Build a `cgeneric` to implement the interaction between
#' two GMRF models each one implemented as a `cgeneric`.
#' @export
setMethod(
  "kronecker",
  c(X="cgeneric", Y = "cgeneric"),
  function(X, Y, FUN = "*", make.dimnames = FALSE, ...) {
    mcall <- match.call()
    if(is.null(mcall$debug)) {
      debug <-
        max(X$f$cgeneric$debug,
            Y$f$cgeneric$debug)
    } else {
      debug <- eval(mcall$debug)
      stopifnot(is.logical(debug))
    }

    if(is.null(mcall$useINLAprecomp)) {
      useINLAprecomp = TRUE
    } else {
      useINLAprecomp = mcall$useINLAprecomp
    }

    INLAvcheck <- packageCheck("INLA", "25-10-28")
    if(is.na(INLAvcheck) & useINLAprecomp) {
      useINLAprecomp <- FALSE
      warning("INLA version is old. Setting 'useINLAprecomp = FALSE'!")
    }
    shlib <- cgeneric_shlib_path(
      package = "INLAtools",
      useINLAprecomp = useINLAprecomp,
      debug = debug)

    cmodel <- "inla_cgeneric_kronecker"

    n1 <- as.integer(X$f$n)
    n2 <- as.integer(Y$f$n)
    N <- as.integer(n1 * n2)

    if(debug) {
      cat('n1:', n1, "n2:", n2, "n:", N, "")
    }

    ## Q1 graph index: i,j
    ij1 <- cgeneric_get(
      model = X,
      cmd = "graph",
      optimize = TRUE
    )
    stopifnot(all(ij1[[1]]<=ij1[[2]]))
    M1 <- length(ij1[[1]])
    if(debug) {
      cat('M1:', M1, "\n")
      print(str(ij1))
    }
    names(ij1) <- c("i", "j")

    ## Q2 graph index: i,j
    ij2 <- cgeneric_get(
      model = Y,
      cmd = "graph",
      optimize = TRUE
    )
    stopifnot(all(ij2[[1]]<=ij2[[2]]))
    M2 <- length(ij2[[1]])
    if(debug) {
      cat('M2:', M2, "\n")
      print(str(ij2))
    }
    names(ij2) <- c('i', 'j')

    ## For Q = Q1 (x) Q2
    ## The total number of elements in Q is
    ##  [(M1-n1)*2+n1]*[(M2-n2)*2+n2]      =
    ##    (2*M1 - n1) * (2*M2 - n2)        =
    ##  n1*n2 + 4*M1*M2 -2*M1*n2 -2*M2*n1  =
    ##  n1*n2 + 2*M1*(2*M2 - n2) - 2*M2*n1
    ## but at the upper part is made up of
    ## diagonal + half_of_the_off_diagonal
    ##  n1*n2   + M1*(2*M2 - n2) - M2*n1         =
    ##  n1*n2   + 2*M1*M2 - M1*n2 - M2*n1        =
    ##  n1*n2   + M1*M2 + M1*M2 - M1*n2 - M2*n1  =
    ##  n1*n2   + M1*M2 + M1*(M2-n2) - M2*n1     =
    ##  n1*n2   + M1*(M2-n2) + M2*(M1-n1)

    ## define u1 = M1-n1, u2 = M2-n2
    ##  n1*n2 + M1*u2        + M2*u1            =
    ##  n1*n2 + (n1 + u1)*u2 + (n2 + u2)*u1     =
    ##  n1*n2 + n1*u2 + u1*u2 + n2*u1 + u2*u1   =
    ##  n1*(n2 + u2)  + u1*(n2 + u2)  + u2*u1   =
    ##  n1*M2         + u1*M2         + u2*u1   =
    ##  (n1 + u1) * M2                + u2*u1   =
    ##         M1 * M2 + u2 * u1
    ## the i<j part from Q1
    idx1u <- which(ij1$i < ij1$j)
    u1 <- length(idx1u)
    stopifnot(u1 == (M1-n1))
    ## the i<j part from Q2
    idx2u <- which(ij2$i < ij2$j)
    u2 <- length(idx2u)
    stopifnot(u2 == (M2-n2))

    ## the number of non-diagonal elements
    M <- M1 * M2 + u1 * u2
    if(debug) {
      cat("u1:", u1, "u2:", "u2", u2, "M:", M, "\n")
    }

    ## resulting graph index i
    ii0 <- rep(ij1$i * n2, each = M2)  + ij2$i
    ## resulting graph index j
    jj0 <- rep(ij1$j * n2, each = M2)  + ij2$j
    if((u1*u2)>0) {
      ii <- c(
        ii0,
        rep(ij1$i[idx1u]*n2, each=u2) + ij2$j[idx2u])
      jj <- c(
        jj0,
        rep(ij1$j[idx1u]*n2, each=u2) + ij2$i[idx2u]
      )
    } else {
      ii <- ii0
      jj <- jj0
    }

    ## check
    stopifnot(all(ii <= jj))

    ## the order of the output should use this
    jjord <- order(jj)
    iiord <- order(ii[jjord])
    ije <- list(
      ii = ii[jjord][iiord],
      jj = jj[jjord][iiord],
      ord = jjord[iiord]
    )

    ## initial data
    ret <- structure(
      list(
        f = list(
          model = "cgeneric",
          n = as.integer(N),
          cgeneric = structure(
            list(
              model = cmodel,
              shlib = shlib,
              n = as.integer(N),
              debug = as.integer(debug)
            ),
            # inla.cgeneric is needed to support INLA before August 2025
            class = c("inla.cgeneric.f", "inla.cgeneric")
          )
        )
      ),
      class = c("cgeneric", "inla.cgeneric")
    )
    ret$f$cgeneric$data <- vector("list", 5L)
    names(ret$f$cgeneric$data) <- c(
      "ints", "doubles", "characters",
      "matrices", "smatrices")

    ## data size for each model
    ndata1 <- sapply(
      X$f$cgeneric$data,
      length)
    ndata2 <- sapply(
      Y$f$cgeneric$data,
      length)

    ### n and the data size info
      Ndata <- list(
        n = as.integer(
          c(n1, ndata1, M1,
                ndata2, M2, N, M)
        )
      )
      if(debug) {
        cat(Ndata$n, "\n")
      }
      ret$f$cgeneric$data$ints <-
        c(
          Ndata,
          ### concatenate ints from each model
          X$f$cgeneric$data$ints[-1], ## n for model 1 is already there
          Y$f$cgeneric$data$ints,
          list(
            idx1u = as.integer(idx1u - 1),
            idx2u = as.integer(idx2u - 1)
          )
        )

      if((ndata1[2]>0) | (ndata2[2]>0)) {
        ret$f$cgeneric$data$doubles <-
          c(
            X$f$cgeneric$data$doubles,
            Y$f$cgeneric$data$doubles
          )
      }

      ret$f$cgeneric$data$characters <-
      c(
        list(
          model = cmodel,
          shlib = shlib
        ),
        X$f$cgeneric$data$characters,
        Y$f$cgeneric$data$characters
      )

      if((ndata1[4]>0) | (ndata2[4]>0)) {
        ret$f$cgeneric$data$matrices <-
          c(
            X$f$cgeneric$data$matrices,
            Y$f$cgeneric$data$matrices
          )
      }

    ret$f$cgeneric$data$smatrices <-
      c(
        X$f$cgeneric$data$smatrices,
        Y$f$cgeneric$data$smatrices,
        list(Kgraph = c(
          N, N, M,
          ije$ii,
          ije$jj,
          as.numeric(ije$ord-1)
          )
        )
      )

    if((!is.null(X$f$extraconstr)) |
       (!is.null(Y$f$extraconstr))) {
        ret$f$extraconstr <-
          kronecker_extraconstr(
            X$f$extraconstr,
            Y$f$extraconstr,
            X$f$n, Y$f$n
          )
        if(debug) {
          cat(nrow(ret$f$extraconstr),
              " 'extraconstr' built!\n")
        }
    }

    # Note: kron(X,Y) gives X-major ordering (the X-index varies slowly, the
    # Y-index varies quickly), which requires the mappers to be in reverse
    # order, multi(Y,X):
    inlabruCheck <- packageCheck("inlabru", "2.13.0.9005")
    if(is.na(inlabruCheck)) {
      warning("Please install a inlabru recent version.")
    } else {
      ret[["mapper"]] <- multi_generic_model_mapper(list(Y, X))
    }

    return(ret)
  }
)
#' @describeIn kronecker
#' Build a `rgeneric` to implement the interaction between two GMRF
#' models, the first as a `cgeneric` and the second as a `rgeneric`.
#' @export
setMethod(
  "kronecker",
  c(X="cgeneric", Y = "rgeneric"),
  function(X, Y, FUN = "*", make.dimnames = FALSE, ...) {
    mcall <- match.call()
    if(is.null(mcall$debug)) {
      debug <-
        max(X$f$cgeneric$debug,
            Y$f$cgeneric$debug)
    } else {
      debug <- eval(mcall$debug)
      stopifnot(is.logical(debug))
    }

    n <- X$f$n * Y$f$n

    ini1 <- cgeneric_get(X, "initial")
    nth1 <- length(ini1)
    ini2 <- rgeneric_get(Y, "initial")
    nth2 <- length(ini2)

    kmodel <- function(cmd = c("graph", "Q", "mu",
                               "initial", "log.norm.const",
                               "log.prior", "quit"),
                       theta = NULL) {

      graph <- function(n, theta) {
        g1 <- cgeneric_get(X, cmd = "graph", optimize = FALSE)
        g2 <- rgeneric_get(Y, cmd = "graph", optimize = FALSE)
        return(Sparse(kronecker(g1, g2)))
      }

      Q <- function(n, theta) {
        Q1 <- cgeneric_get(X, "Q", theta = theta[1:nth1], optimize = FALSE)
        Q2 <- rgeneric_get(Y, "Q", theta = theta[nth1+1:nth2], optimize = FALSE)
        QQ <- Sparse(kronecker(Q1, Q2))
        idx <- which(QQ@i <= QQ@j)
        return(QQ@x[idx])
      }

      mu <- function(n, theta)
        return(numeric(0))

      log.norm.const <- function(n, theta)
        return(numeric(0))

      log.prior <- function(n, theta) {
        return(
          cgeneric_get(X, cmd = "log_prior", theta = theta[1:nth1]) +
            rgeneric_get(Y, cmd = "log_prior", theta = theta[nth1+1:nth2])
        )
      }

      initial <- function(n, theta) {
        return(
          c(ini1, ini2)
        )
      }

      quit <- function(n, theta) {
        return(invisible())
      }

      cmd <- match.arg(cmd)

      ret <- do.call(
        cmd,
        args = list(n = n,
                    theta = theta
        )
      )

      return(ret)

    }

    ### follows INLA:::inla.rgeneric.define() but no assign env
    rmodel <- structure(
      list(
        f = list(
          model = "rgeneric",
          n = n,
          rgeneric = structure(
            list(
              definition =
                compiler::cmpfun(
                  kmodel,
                  options = list(optimize = 3L)),
              debug = debug,
              optimize = TRUE
            ),
            # inla.rgeneric is needed to support INLA before August 2025
            class = c("inla.rgeneric.f", "inla.rgeneric")
          )
        )
      ),
      class = c("rgeneric", "inla.rgeneric")
    )

    if((!is.null(X$f$extraconstr)) |
       (!is.null(Y$f$extraconstr))) {
      rmodel$f$extraconstr <-
        kronecker_extraconstr(
          X$f$extraconstr,
          Y$f$extraconstr,
          X$f$n, Y$f$n
        )
      if(debug) {
        cat(nrow(rmodel$f$extraconstr),
            " 'extraconstr' built\n")
      }
    }

    # Note: kron(X,Y) gives X-major ordering (the X-index varies slowly, the
    # Y-index varies quickly), which requires the mappers to be in reverse
    # order, multi(Y,X):
    rmodel$mapper <- multi_generic_model_mapper(list(Y, X))

    return(rmodel)
  }
)
#' @describeIn kronecker
#' Build a `rgeneric` to implement the interaction between two GMRF
#' models, the first as a `rgeneric` and the second as a `cgeneric`.
#' @export
setMethod(
  "kronecker",
  c(X="rgeneric", Y = "cgeneric"),
  function(X, Y, FUN = "*", make.dimnames = FALSE, ...) {

    mcall <- match.call()
    if(is.null(mcall$debug)) {
      debug <-
        max(X$f$cgeneric$debug,
            Y$f$cgeneric$debug)
    } else {
      debug <- eval(mcall$debug)
      stopifnot(is.logical(debug))
    }

    n <- X$f$n * Y$f$n

    ini1 <- rgeneric_get(X, "initial")
    nth1 <- length(ini1)
    ini2 <- cgeneric_get(Y, "initial")
    nth2 <- length(ini2)

    kmodel <- function(cmd = c("graph", "Q", "mu",
                               "initial", "log.norm.const",
                               "log.prior", "quit"),
                       theta = NULL) {

      graph <- function(n, theta) {
        g1 <- rgeneric_get(X, cmd = "graph", optimize = FALSE)
        g2 <- cgeneric_get(Y, cmd = "graph", optimize = FALSE)
        return(Sparse(kronecker(g1, g2)))
      }

      Q <- function(n, theta) {
        Q1 <- rgeneric_get(X, cmd = "Q", theta = theta[1:nth1], optimize = FALSE)
        Q2 <- cgeneric_get(Y, cmd = "Q", theta = theta[nth1+1:nth2], optimize = FALSE)
        QQ <- Sparse(kronecker(Q1, Q2))
        idx <- which(QQ@i <= QQ@j)
        return(QQ@x[idx])
      }

      mu <- function(n, theta)
        return(numeric(0))

      log.norm.const <- function(n, theta)
        return(numeric(0))

      log.prior <- function(n, theta) {
        return(
          rgeneric_get(X, cmd = "log_prior", theta = theta[1:nth1]) +
          cgeneric_get(Y, cmd = "log_prior", theta = theta[nth1+1:nth2])
        )
      }

      initial <- function(n, theta) {
        return(c(ini1, ini2))
      }

      quit <- function(n, theta) {
        return(invisible())
      }

      cmd <- match.arg(cmd)

      ret <- do.call(
        cmd,
        args = list(n = n,
                    theta = theta
        )
      )

      return(ret)

    }

    ### follows INLA:::inla.rgeneric.define() but no assign env
    rmodel <- structure(
      list(
        f = list(
          model = "rgeneric",
          n = n,
          rgeneric = structure(
            list(
              definition =
                compiler::cmpfun(
                  kmodel,
                  options = list(optimize = 3L)),
              debug = debug,
              optimize = TRUE
            ),
            # inla.rgeneric is needed to support INLA before August 2025
            class = c("inla.rgeneric.f", "inla.rgeneric")
          )
        )
      ),
      class = c("rgeneric", "inla.rgeneric")
    )

    if((!is.null(X$f$extraconstr)) |
       (!is.null(Y$f$extraconstr))) {
      rmodel$f$extraconstr <-
        kronecker_extraconstr(
          X$f$extraconstr,
          Y$f$extraconstr,
          X$f$n, Y$f$n
        )
      if(debug) {
        cat(nrow(rmodel$f$extraconstr),
            " 'extraconstr' built!\n")
      }
    }

    # Note: kron(X,Y) gives X-major ordering (the X-index varies slowly, the
    # Y-index varies quickly), which requires the mappers to be in reverse
    # order, multi(Y,X):
    rmodel$mapper <- multi_generic_model_mapper(list(Y, X))

    return(rmodel)
  }
)
#' @describeIn kronecker
#' Build a `rgeneric` to implement the interaction between
#' two GMRF models each one implemented as a `rgeneric`.
setMethod(
  "kronecker",
  c(X="rgeneric", Y = "rgeneric"),
  function(X, Y, FUN = "*", make.dimnames = FALSE, ...) {

    mcall <- match.call()
    if(is.null(mcall$debug)) {
      debug <-
        max(X$f$cgeneric$debug,
            Y$f$cgeneric$debug)
    } else {
      debug <- eval(mcall$debug)
      stopifnot(is.logical(debug))
    }

    n <- X$f$n * Y$f$n

    ini1 <- rgeneric_get(X, "initial")
    nth1 <- length(ini1)
    ini2 <- rgeneric_get(Y, "initial")
    nth2 <- length(ini2)

    kmodel <- function(cmd = c("graph", "Q", "mu",
                               "initial", "log.norm.const",
                               "log.prior", "quit"),
                       theta = NULL) {

      graph <- function(n, theta) {
        g1 <- rgeneric_get(X, cmd = "graph", optimize = FALSE)
        g2 <- rgeneric_get(Y, cmd = "graph", optimize = FALSE)
        return(Sparse(kronecker(g1, g2)))
      }

      Q <- function(n, theta) {
        Q1 <- rgeneric_get(X, cmd = "Q", theta = theta[1:nth1], optimize = FALSE)
        Q2 <- rgeneric_get(Y, cmd = "Q", theta = theta[nth1+1:nth2], optimize = FALSE)
        QQ <- Sparse(kronecker(Q1, Q2))
        idx <- which(QQ@i <= QQ@j)
        return(QQ@x[idx])
      }

      mu <- function(n, theta)
        return(numeric(0))

      log.norm.const <- function(n, theta)
        return(numeric(0))

      log.prior <- function(n, theta) {
        lp1 <- rgeneric_get(X, cmd = "log.prior", theta = theta[1:nth1])
        lp2 <- rgeneric_get(Y, cmd = "log.prior", theta = theta[nth1+1:nth2])
        return(lp1 + lp2)
      }

      initial <- function(n, theta) {
        return(c(ini1, ini2))
      }

      quit <- function(n, theta) {
        return(invisible())
      }

      cmd <- match.arg(cmd)

      ret <- do.call(
        cmd,
        args = list(
          n = n,
          theta = theta
        )
      )
      return(ret)
    }

    rmodel <- structure(
      list(
        f = list(
          model = "rgeneric",
          n = n,
          rgeneric = structure(
            list(
              definition =
                compiler::cmpfun(
                  kmodel,
                  options = list(optimize = 3L)),
              debug = debug,
              optimize = TRUE
            ),
            # inla.rgeneric is needed to support INLA before August 2025
            class = c("inla.rgeneric.f", "inla.rgeneric")
          )
        )
      ),
      class = c("rgeneric", "inla.rgeneric")
    )

    if((!is.null(X$f$extraconstr)) |
       (!is.null(Y$f$extraconstr))) {
      rmodel$f$extraconstr <-
        kronecker_extraconstr(
          X$f$extraconstr,
          Y$f$extraconstr,
          X$f$n, Y$f$n
        )
      if(debug) {
        cat(nrow(rmodel$f$extraconstr),
            " 'extraconstr' built!\n")
      }
    }

    # Note: kron(X,Y) gives X-major ordering (the X-index varies slowly, the
    # Y-index varies quickly), which requires the mappers to be in reverse
    # order, multi(Y,X):
    rmodel$mapper <- multi_generic_model_mapper(list(Y, X))

    return(rmodel)
  }
)
#' Kronecker (product) between `extraconstr`,
#' implemented for [kronecker()] methods.
#' @name extraconstr
#' @param c1,c2 named list with two elements:
#' `A` and `e`, where `nrow(A)` should be equal
#' to `length(e)`. These are constraint definitions.
#' @param n1,n2 integer with each model's length.
#' @returns The constraint definition for the
#' whole latent model built from the Kronecker product.
#' A length two named list. 'A' a matrix and
#' 'e' a vector where nrow(A)=length(e) and
#' ncol(A)=(n1*n2).
kronecker_extraconstr <- function(c1, c2, n1, n2) {
  if(is.null(c1)) {
    if(is.null(c2)) {
      return(NULL)
    } else {
      stopifnot(ncol(c2$A) == n2)
      ret <- list(
        A = kronecker(diag(n1), c2$A),
        e = rep(c2$e, n1)
      )
    }
  } else {
    stopifnot(ncol(c1$A) == n1)
    if(is.null(c2)) {
      ret <- list(
        A = kronecker(c1$A, diag(n2)),
        e = rep(c1$e, each = n2)
      )
    } else {
      stopifnot(ncol(c2$A) == n2)
      nc1 <- nrow(c1$A)
      nc2 <- nrow(c2$A)
      Aret <- rbind(
        kronecker(c1$A, diag(n2)),
        kronecker(diag(n1), c2$A)
      )
      eret <- c(rep(c1$e, each = n2),
                rep(c2$e, n1))
      ## remove redundant constraints
      AAtsvd <- svd(tcrossprod(Aret))
      jj <- which(AAtsvd$d>sqrt(.Machine$double.eps))
      ret <- list(
        A = Aret[jj, , drop = FALSE],
        e = eret[jj]
      )
    }
  }
  return(ret)
}


#' @title Combine two or more `cgeneric` or `rgeneric` models
#' @description Constructs a multiple kronecker product model from a list of model
#' objects. The resulting model contains a corresponding [inlabru::bm_multi()]
#' mapper. This can be used as an alternative to a binary tree of kronecker
#' product models.
#' @param models A list of `cgeneric` or `rgeneric` models, optionally with names
#' @details The last model in the list has the slowest index variation, and the
#'   first model has the fastest index variation. This matches the latent
#'   variable ordering of standard `INLA:f()` model components with
#'   `(main, group, replicate)`.
#' @param \dots Arguments passed on to every `kronecker()` call.
#' @returns A 'cgeneric' or 'rgeneric' model object, containing a
#'   multi-kronecker product model, with a corresponding [inlabru::bm_multi()]
#'   mapper.
#' @rdname multi_generic_model
#' @export
multi_generic_model <- function(models, ...) {
  stopifnot(is.list(models))
  stopifnot(length(models) >= 1L)

  models <- lapply(models, function(model) {
    if (inherits(model, c("rgeneric", "inla.rgeneric"))) {
      rgeneric(model)
    } else if (inherits(model, c("cgeneric", "inla.cgeneric"))) {
      cgeneric(model)
    } else {
      stop("Each 'models' element must be convertible to class 'cgeneric' or 'rgeneric'")
    }
  })

  ret <- models[[1]]
  for (i in seq_along(models)[-1]) {
    ret <- Matrix::kronecker(models[[i]], ret, ...)
  }

  inlabruCheck <- packageCheck("inlabru", "2.13.0.9005")
  if(is.na(inlabruCheck)) {
    warning("Please install a inlabru recent version.")
  } else {
    ret[["mapper"]] <- multi_generic_model_mapper(models)
  }
  return(ret)
}
#' @describeIn multi_generic_model
#' Buidl the bm_multi mapper for a list of models
multi_generic_model_mapper <- function(models) {
  vs <- "2.13.0.9005"
  inlabruCheck <- packageCheck(
    name = "inlabru",
    minimum_version = vs) >= vs
  mapps <- lapply(seq_along(models), function(i)
    mapper1(models[[i]]))
  if(!is.na(inlabruCheck)) {
    fn <- findGetFunction("bm_multi", "inlabru")
    return(fn(mapps))
  } else {
    mmap <- list(n = prod(sapply(mapps, function(x) x$n)))
    class(mmap) <- "bm_index"
    return(mmap)
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pc_utils.R
\name{pc-utils}
\alias{pc-utils}
\alias{pcParamCheck}
\alias{pclrange}
\alias{pcrange}
\title{Internal functions to check PC-prior parameters.}
\usage{
pcParamCheck(npars, reference, probability)

pclrange(lrange, lam, d = 2, log = FALSE)

pcrange(range, lam, d = 2, log = FALSE)
}
\arguments{
\item{npars}{number of parameters.}

\item{reference}{numeric vector to set the reference
for each parameter for its PC-prior.}

\item{probability}{numeric vector with to
set the probability statement of the PC prior for each
parameter. For \code{sigma} probability statement is
P(sigma > \code{reference}) = p whereas for \code{range} it is
P(range < \code{reference}). If  NA, 0 or 1, the corresponding
\code{reference} will be used as fixed. If missing, all the
parameters considered as known (fixed) and equal the
corresponding reference value.}

\item{lrange}{numeric with the log of the (practical) range}

\item{lam}{numeric with the prior parameter}

\item{d}{integer to specify the domain dimention}

\item{log}{logical indicating if the density
is to be returned in the log scale}

\item{range}{numeric with the of the (practical) range.}
}
\description{
Internal functions to check PC-prior parameters.
}
\section{Functions}{
\itemize{
\item \code{pcParamCheck()}: Check the PC-prior arguments.

\item \code{pclrange()}: Penalized Complexity (PC) prior for the log of the practical range.

\item \code{pcrange()}: Penalized Complexity (PC) prior for the practical range.

}}
\examples{
# P(range < 2.0) = 0.1
 lam <- -log(0.1) * 2.0
 plot(function(x) pcrange(x, lam), 1/100, 10, n = 100)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Matrix.R
\name{upperPadding}
\alias{upperPadding}
\title{Padding (a list of) sparse matrices.}
\usage{
upperPadding(M, relative = FALSE, ...)
}
\arguments{
\item{M}{'Matrix' (or a list of them).}

\item{relative}{logical. If 'M" is a list,
it indicates if it is to be returned a relative index
and the value for each matrix. See details.}

\item{...}{additional arguments passed to \link{Sparse}.}
}
\value{
If a unique matrix is given, return the
upper triangle considering the 'T' representation
in the \code{dgTMatrix}, from the \code{Matrix} package.
If a list of matrices is given,
return a list of two elements: 'graph' and 'xx'.
The 'graph' is the union of the graph from each matrix.
If relative=FALSE, 'xx' is a matrix with number of column equals
the the number of matrices imputed.
If relative=TRUE, it is a list of length equal the number
of matrices imputed. See details.
}
\description{
Padding (a list of) sparse matrices.
}
\details{
This is useful to prepare a matrix, or a list of,
sparse matrices for use in some 'cgeneric' code.

Define a graph of the union of the supplied matrices
and return the row ordered diagonal plus upper triangle
after padding with zeroes each one so that
all the returned matrices have the same pattern.

If relative=FALSE, each columns of 'xx' is the
elements of the corresponding matrix after being padded
to fill the pattern of the union graph.
If relative=TRUE, each element of 'xx' would be a list
with a relative index, 'r', for each non-zero elements of
each matrix is returned relative to the union graph,
the non-lower elements, 'x', of the corresponding matrix,
and a vector, 'o', with the number of non-zero elements
for each line of each resulting matrix.
}
\examples{
A <- sparseMatrix(
  i = c(1, 1, 2, 3, 3, 5),
  j = c(2, 5, 3, 4, 5, 5),
  x = -c(0:3,NA,1), symmetric = TRUE)
A
upperPadding(A)
upperPadding(A, na.rm = TRUE)
upperPadding(A, zeros.rm = TRUE)
upperPadding(A, na.rm = TRUE, zeros.rm = TRUE)
B <- Diagonal(nrow(A), -colSums(A, na.rm = TRUE))
B
upperPadding(list(a = A, b = B), na.rm = TRUE, zeros.rm = TRUE)
upperPadding(list(a = A, b = B), relative = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{findGetFunction}
\alias{findGetFunction}
\title{Search a function and retrieve it.}
\usage{
findGetFunction(fName, package, debug = FALSE)
}
\arguments{
\item{fName}{character with the name of the function}

\item{package}{character with the package name}

\item{debug}{logical indicating if it is to print
intermediate progress finding}
}
\value{
function.
The (first) package name where it was found
is returned as an attribute named "package"
}
\description{
Search a function and retrieve it.
}
\details{
if 'missing(package)' it will search on the loaded
packages, first in the exported functions, and then
among the non-exported ones.
NOTE: 'package' can include any installed package,
see \code{\link[=installed.packages]{installed.packages()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aaa_methods.R
\name{INLAtools-methods}
\alias{INLAtools-methods}
\alias{initial}
\alias{mu}
\alias{prior}
\alias{graph}
\alias{prec}
\alias{prec.default}
\alias{vcov.Matrix}
\title{Methods to work with a \code{model}.}
\usage{
initial(model)

mu(model, theta)

prior(model, theta)

graph(model, optimize)

prec(model, theta, optimize)

\method{prec}{default}(model, ...)

\method{vcov}{Matrix}(object, ...)
}
\arguments{
\item{model}{object to represent a model}

\item{theta}{numeric vector.
For \code{prior} it can be a numeric matrix,
with number of lines equal the size of \code{theta}
and each column as a different case.}

\item{optimize}{logical indicating if it is to be
returned only the elements and not as a sparse matrix.}

\item{...}{additional arguments passed on}

\item{object}{Matrix supposed to be a
sparse precision matrix}
}
\value{
the result of the desired query
of the 'cgeneric' model.
'graph' and 'prec' can be either a vector
(if optimize = TRUE) or a sparse matrix.
}
\description{
For a given model object query the \code{initial},
\code{mu}, log \code{prior}, \code{graph} or precision \code{prec}
can be evaluated/retrieved.
}
\section{Functions}{
\itemize{
\item \code{initial()}: Retrieve the initial model parameter(s)

\item \code{mu()}: Evaluate the model's mean

\item \code{prior()}: Evaluate the log-prior for a given \code{theta}

\item \code{graph()}: Retrieve the models' graph

\item \code{prec()}: Retrieve the precision for a given \code{theta}

\item \code{prec(default)}: The default precision method
computes the inverse of the variance

\item \code{vcov(Matrix)}: The \code{vcov} method for sparse matrices

}}
\seealso{
\code{\link[=prior.cgeneric]{prior.cgeneric()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kronecker.R
\name{multi_generic_model}
\alias{multi_generic_model}
\alias{multi_generic_model_mapper}
\title{Combine two or more \code{cgeneric} or \code{rgeneric} models}
\usage{
multi_generic_model(models, ...)

multi_generic_model_mapper(models)
}
\arguments{
\item{models}{A list of \code{cgeneric} or \code{rgeneric} models, optionally with names}

\item{\dots}{Arguments passed on to every \code{kronecker()} call.}
}
\value{
A 'cgeneric' or 'rgeneric' model object, containing a
multi-kronecker product model, with a corresponding \code{\link[inlabru:bm_multi]{inlabru::bm_multi()}}
mapper.
}
\description{
Constructs a multiple kronecker product model from a list of model
objects. The resulting model contains a corresponding \code{\link[inlabru:bm_multi]{inlabru::bm_multi()}}
mapper. This can be used as an alternative to a binary tree of kronecker
product models.
}
\details{
The last model in the list has the slowest index variation, and the
first model has the fastest index variation. This matches the latent
variable ordering of standard \code{INLA:f()} model components with
\verb{(main, group, replicate)}.
}
\section{Functions}{
\itemize{
\item \code{multi_generic_model_mapper()}: Buidl the bm_multi mapper for a list of models

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aaa_classes.R, R/cgeneric.R
\docType{class}
\name{cgeneric-class}
\alias{cgeneric-class}
\alias{cgeneric}
\alias{cgeneric.character}
\alias{cgeneric.function}
\alias{cgeneric.cgeneric}
\alias{cgeneric.inla.cgeneric}
\alias{cgenericBuilder}
\alias{mapper1}
\alias{cgeneric_shlib_path}
\alias{print.cgeneric}
\alias{summary.cgeneric}
\alias{plot.cgeneric}
\title{Organize data for the latent GMRF C interface for \code{INLA}.}
\usage{
cgeneric(model, ...)

\method{cgeneric}{character}(model, ...)

\method{cgeneric}{`function`}(model, ...)

\method{cgeneric}{cgeneric}(model, ...)

\method{cgeneric}{inla.cgeneric}(model, ...)

cgenericBuilder(...)

mapper1(model)

cgeneric_shlib_path(package, useINLAprecomp, debug)

\method{print}{cgeneric}(x, ...)

\method{summary}{cgeneric}(object, ...)

\method{plot}{cgeneric}(x, y, ...)
}
\arguments{
\item{model}{object class for what a \code{cgeneric} method exists.
if it is a character, a specific function will be called,
for example cgeneric("iid", ...") calls cgeneric_iid(...).}

\item{...}{additional arguments passed on to methods
It should include
\code{n} and \code{debug}. For \code{cgenericBuild} and
\code{model} as a character string with the name of the
C function and \code{shlib} as the path to the
shared object containing such function.
If \code{shlib} is not provided it can be built
using \code{cgeneric_shlib_path} for the
\code{package} (character with the R package containing it),
in which \code{useINLAprecomp} (logical) indicates if it is to
use the one contained within \code{INLA} or in the package.
When using the inlabru package,
one can also provide a \code{mapper} which will be evaluated by
the bru_get_mapper inlabru's function.}

\item{package}{character giving the name of the package
that contains the \code{cgeneric} model.}

\item{useINLAprecomp}{logical, indicating if it is to use
the shared object previously copied and compiled by INLA.}

\item{debug}{integer, used as verbose in debug.}

\item{x}{a cgeneric object}

\item{object}{a cgeneric object}

\item{y}{not used}
}
\value{
a method to build a \code{cgeneric} should return
a named list of \code{cgeneric} class that contains a
named list \code{f} that contains (at least):
\itemize{
\item \code{model} a character always equal to \code{cgeneric},
\item \code{n} an integer greater than 0, and
\item \code{cgeneric} as a named list that contains the
data needed to define the model. Each element on
...$f$cgeneric is also a named list containing
\code{ints}, \code{doubles}, \code{characters}, \code{matrices}
and \code{smatrices}.
\item (possible) \code{extraconstr} as a named list with: \code{A} as a
\code{n} times \code{k} matrix and \code{e} as a length \code{k} vector.
}

The \code{cgeneric_shlib_path} function returns a \code{character}
with the path to the shared lib.
}
\description{
A GMRF is defined from model parameters \eqn{\theta} that
would parametrize a (sparse) precision matrix.

The elements of a GMR are:
\itemize{
\item \code{graph} to define the non-zero precision matrix pattern.
only the upper triangle including the diagonal is needed.
\item \code{Q} vector where the
\itemize{
\item first element (N) is the size of the matrix,
\item second element (M) is the number of non-zero
elements in the upper part (including) diagonal
\item the remaining (M) elements are the actual
precision (upper triangle plus diagonal) elements
whose order shall follow the graph definition.
}
\item \code{mu} the mean vector,
\item \code{initial} vector with
\itemize{
\item first element as the number of the parameters in the model
\item remaining elements should be the initials for the model parameters.
}
\item \code{log.norm.const} log of the normalizing constant.
\item \code{log.prior} log of the prior for the model parameters.
}
}
\section{Functions}{
\itemize{
\item \code{cgeneric(character)}: Call a function named as \code{cgeneric_[model]} to build a \code{cgeneric}.

\item \code{cgeneric(`function`)}: The \code{cgeneric} method for function.

\item \code{cgeneric(cgeneric)}: Check, append \code{cgeneric} class, returns model unchanged.

\item \code{cgeneric(inla.cgeneric)}: Check and converts a regular \code{inla.cgeneric} object to \code{cgeneric}.

\item \code{cgenericBuilder()}: Build a \code{cgeneric} from a list of arguments.

\item \code{mapper1()}: A default mapper for a cgeneric/rgeneric model

\item \code{cgeneric_shlib_path()}: Make the lib path for the shared lib of a package.

\item \code{print(cgeneric)}: Print the cgeneric object

\item \code{summary(cgeneric)}: A summary for a cgeneric object

\item \code{plot(cgeneric)}: A plot for a cgeneric object

}}
\note{
The \code{graph} and \code{Q} non-zero pattern should match,
its elements should be ordered by row,
and only its upper part stored.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aaa_methods.R
\name{is.zero}
\alias{is.zero}
\alias{is.zero.default}
\alias{is.zero.matrix}
\alias{is.zero.Matrix}
\title{Define the is.zero method}
\usage{
is.zero(x, tol)

\method{is.zero}{default}(x, tol)

\method{is.zero}{matrix}(x, tol)

\method{is.zero}{Matrix}(x, tol)
}
\arguments{
\item{x}{an R object}

\item{tol}{numeric to be used as (absolute) tolerance.
if missing (default) it will consider \code{x==0}.}
}
\value{
logical
}
\description{
Define the is.zero method
}
\section{Methods (by class)}{
\itemize{
\item \code{is.zero(default)}: The is.zero.default definition

\item \code{is.zero(matrix)}: The is.zero.matrix definition

\item \code{is.zero(Matrix)}: The is.zero.Matrix definition

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{packageCheck}
\alias{packageCheck}
\title{To check package version and load}
\usage{
packageCheck(name, minimum_version, quietly = FALSE)
}
\arguments{
\item{name}{character with the name of the package}

\item{minimum_version}{character with the minimum required version}

\item{quietly}{logical indicating if messages shall be printed}
}
\description{
To check package version and load
}
\note{
Original in inlabru package function check_package_version_and_load
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inla_methods.R
\name{prec.inla}
\alias{prec.inla}
\title{Define the prec method for an inla output object}
\usage{
\method{prec}{inla}(model, ...)
}
\arguments{
\item{model}{an inla output}

\item{...}{used to pass the 'prior' argument,
as logical (default is TRUE) to indicate if it is
to retrieve the prior or the posterior precision.}
}
\description{
Define the prec method for an inla output object
}
\details{
extract the joint prior precision for the
latent field at the mode of the hyperparameters
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kronecker.R
\name{kronecker}
\alias{kronecker}
\alias{kronecker,cgeneric,cgeneric-method}
\alias{kronecker,cgeneric,rgeneric-method}
\alias{kronecker,rgeneric,cgeneric-method}
\alias{kronecker,rgeneric,rgeneric-method}
\title{Kronecker between \code{cgeneric}|\code{rgeneric} to implement interaction
between GMRF models.}
\usage{
\S4method{kronecker}{cgeneric,cgeneric}(X, Y, FUN = "*", make.dimnames = FALSE, ...)

\S4method{kronecker}{cgeneric,rgeneric}(X, Y, FUN = "*", make.dimnames = FALSE, ...)

\S4method{kronecker}{rgeneric,cgeneric}(X, Y, FUN = "*", make.dimnames = FALSE, ...)

\S4method{kronecker}{rgeneric,rgeneric}(X, Y, FUN = "*", make.dimnames = FALSE, ...)
}
\arguments{
\item{X}{\code{cgeneric} or \code{rgeneric}}

\item{Y}{\code{cgeneric} or \code{rgeneric}}

\item{FUN}{see \code{\link[=kronecker]{kronecker()}}}

\item{make.dimnames}{see \code{\link[=kronecker]{kronecker()}}}

\item{...}{see \code{\link[=kronecker]{kronecker()}}}
}
\value{
A \code{cgeneric} object if kronecker between two \code{cgeneric},
otherwise a \code{rgeneric} object.
}
\description{
Kronecker between \code{cgeneric}|\code{rgeneric} to implement interaction
between GMRF models.
}
\section{Functions}{
\itemize{
\item \code{kronecker(X = cgeneric, Y = cgeneric)}: Build a \code{cgeneric} to implement the interaction between
two GMRF models each one implemented as a \code{cgeneric}.

\item \code{kronecker(X = cgeneric, Y = rgeneric)}: Build a \code{rgeneric} to implement the interaction between two GMRF
models, the first as a \code{cgeneric} and the second as a \code{rgeneric}.

\item \code{kronecker(X = rgeneric, Y = cgeneric)}: Build a \code{rgeneric} to implement the interaction between two GMRF
models, the first as a \code{rgeneric} and the second as a \code{cgeneric}.

\item \code{kronecker(X = rgeneric, Y = rgeneric)}: Build a \code{rgeneric} to implement the interaction between
two GMRF models each one implemented as a \code{rgeneric}.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inla_methods.R
\name{inlaQ}
\alias{inlaQ}
\title{Define the method to extract the precision from an inla output object.}
\usage{
inlaQ(model, ...)
}
\arguments{
\item{model}{an inla output}

\item{...}{used to pass the 'prior' argument,
as logical (default is TRUE) to indicate if it is
to retrieve the prior or the posterior precision.}
}
\description{
Define the method to extract the precision from an inla output object.
}
\details{
extract the joint prior precision for the
latent field at the mode of the hyperparameters
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgeneric_get.R
\name{rgeneric_get}
\alias{rgeneric_get}
\alias{rgeneric_initial}
\alias{rgeneric_mu}
\alias{rgeneric_graph}
\alias{rgeneric_Q}
\alias{rgeneric_prior}
\title{\code{rgeneric_get} is an internal function used to query
\code{graph}, \code{Q}, \code{initial}, \code{mu} or \code{prior} from a \code{rgeneric}.}
\usage{
rgeneric_get(
  model,
  cmd = c("graph", "Q", "initial", "mu", "log_prior"),
  theta,
  ...
)

rgeneric_initial(model)

rgeneric_mu(model, theta)

rgeneric_graph(model, optimize)

rgeneric_Q(model, theta, optimize)

rgeneric_prior(model, theta)
}
\arguments{
\item{model}{a \code{rgeneric} object.}

\item{cmd}{an string to specify which model element to get}

\item{theta}{numeric vector with the model parameters.
If missing, the \code{initial} will be used.}

\item{...}{additional arguments passed on to methods.
E.g.: \code{optimize = FALSE} return the graph and precision
as a sparse matrix whereas \code{optimize = TRUE} retur the
graph as arow/col indexes and the precision as a numeric
vector with its elements.}

\item{optimize}{logical indicating if it is to be
returned only the elements and not as a sparse matrix.}
}
\value{
depends on \code{cmd}

numeric scalar (if numeric vector is provided
for theta) or vector (if numeric matrix is provided
for theta).
}
\description{
The \code{generic_get} retrieve a model property specified by
\code{cmd} on an \code{rgeneric} object.
The functions listed below are for each \code{cmd} case.
}
\section{Functions}{
\itemize{
\item \code{rgeneric_initial()}: Retrive the initial parameter(s) of an \code{rgeneric} model.

\item \code{rgeneric_mu()}: Evaluate the mean for an \code{rgeneric} model.

\item \code{rgeneric_graph()}: Retrieve the graph of an \code{rgeneric} object

\item \code{rgeneric_Q()}: Retrieve the precision of an \code{rgeneric} object

\item \code{rgeneric_prior()}: Evaluate the prior for an \code{rgeneric} model

}}
\examples{
library(INLAtools)
old.par <- par(no.readonly = TRUE)

## Setting the prior parameters
prior.par <- c(1, 0.5) # P(sigma > 1) = 0.5
cmodel <- cgeneric(
  model = "iid", n = 10,
  param = prior.par)

## prior summaries: sigma and log-precision
(lamb <- -log(prior.par[2])/prior.par[1])
(smedian <- qexp(0.5, lamb))
(smean <- 1/lamb)

## mode: at the minimum of - log-prior
(lpmode <- optimize(function(x)
  -cgeneric_prior(cmodel, theta = x),
  c(-10, 30))$minimum)
## mean: integral of x*f(x)dx
(lpmean <- integrate(function(x)
  exp(cgeneric_prior(cmodel, theta = matrix(x, 1)))*x,
  -10, 30)$value)

## prior visualization: log(precision) and sigma
par(mfrow = c(1, 2))
plot(function(x)
 exp(cgeneric_prior(cmodel, theta = matrix(x, nrow=1))),
  -3, 3, n = 601, xlab = "log-precision",
  ylab = "density")
abline(v = lpmode, lwd = 3, col = 2)
rug(-2*log(smedian), lwd = 3, col = 3)
rug(lpmean, lwd = 3, col = 4)
plot(function(x)
 exp(cgeneric_prior(cmodel,
  theta = matrix(
    -2*log(x),
    nrow = 1))+log(2)-log(x)),
  1/100, 10, n = 1000,
  xlab = expression(sigma),
  ylab = "density")
plot(function(x) dexp(x, lamb),
   1/100, 10, n = 1000,
   add = TRUE, lty = 2, col = 2)
rug(smedian, lwd = 3, col = 3)
rug(smean, lwd = 3, col = 4)

par(old.par)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cgeneric_generic0.R
\name{cgeneric_generic0}
\alias{cgeneric_generic0}
\alias{cgeneric_iid}
\title{Build a \code{cgeneric} object for a \code{generic0} model.}
\usage{
cgeneric_generic0(R, param, constr = TRUE, scale = TRUE, ...)

cgeneric_iid(n, param, constr = FALSE, ...)
}
\arguments{
\item{R}{the structure matrix for the model definition.}

\item{param}{length two vector with the parameters
\code{a} and \code{p} for the PC-prior distribution defined from
\deqn{P(\sigma > a) = p}
where \eqn{\sigma} can be interpreted as marginal standard
deviation of the process if scale = TRUE. See details.}

\item{constr}{logical indicating if it is to add a
sum-to-zero constraint. Default is TRUE.}

\item{scale}{logical indicating if it is to scale
the model. See detais.}

\item{...}{arguments (debug,useINLAprecomp,shlib)
passed on to \code{cgeneric}.}

\item{n}{integer required to specify the model size}
}
\value{
a \code{cgeneric} object, see \code{\link[=cgeneric-class]{cgeneric-class()}}.
}
\description{
Build data needed to implement a model whose
precision has a conditional precision parameter.
This uses the C interface in the 'INLA' package,
that can be used as a linear predictor
model component with an 'f' term.
}
\details{
The precision matrix is defined as
\deqn{Q = \tau R}
where the structure matrix R is supplied by the user
and \eqn{\tau} is the precision parameter.
Following Sørbie and Rue (2014), if scale = TRUE
the model is scaled so that
\deqn{Q = \tau s R}
where \eqn{s} is the geometric mean of the diagonal
elements of the generalized inverse of \eqn{R}.
\deqn{s = \exp{\sum_i \log((R^{-})_{ii})/n}}
If the model is scaled, the geometric mean of the
marginal variances, the diagonal of \eqn{Q^{-1}}, is one.
Therefore, when the model is scaled,
\eqn{\tau} is the marginal precision,
otherwise \eqn{\tau} is the conditional precision.
}
\section{Functions}{
\itemize{
\item \code{cgeneric_iid()}: The \link{cgeneric_iid} uses the \link{cgeneric_generic0}
with the structure matrix as the identity.

}}
\references{
Sigrunn Holbek Sørbye and Håvard Rue (2014).
Scaling intrinsic Gaussian Markov random field priors in
spatial modelling. Spatial Statistics, vol. 8, p. 39-51.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aaa_classes.R, R/rgeneric.R
\docType{class}
\name{rgeneric-class}
\alias{rgeneric-class}
\alias{rgeneric}
\alias{rgeneric.default}
\alias{rgeneric.rgeneric}
\alias{rgeneric.inla.rgeneric}
\alias{print.rgeneric}
\alias{summary.rgeneric}
\alias{plot.rgeneric}
\title{Organize data for the latent GMRF R interface for \code{INLA}.}
\usage{
rgeneric(model, n, debug = FALSE, ...)

\method{rgeneric}{default}(model, n, debug = FALSE, ...)

\method{rgeneric}{rgeneric}(model, ...)

\method{rgeneric}{inla.rgeneric}(model, ...)

\method{print}{rgeneric}(x, ...)

\method{summary}{rgeneric}(object, ...)

\method{plot}{rgeneric}(x, y, ...)
}
\arguments{
\item{model}{an object used to define the model.
See the 'rgeneric' vignette from the INLA package.}

\item{n}{integer with the dimension of the model}

\item{debug}{logical indicating debug state.}

\item{...}{not used}

\item{x}{a rgeneric object}

\item{object}{a rgeneric object}

\item{y}{not used}
}
\value{
\code{rgeneric}/ \code{inla.rgeneric} object.
}
\description{
A GMRF is defined from model parameters \eqn{\theta} that
would parametrize a (sparse) precision matrix.

The elements of a GMR are:
\itemize{
\item \code{graph} to define the non-zero precision matrix pattern.
only the upper triangle including the diagonal is needed.
\item \code{Q} vector where the
\itemize{
\item first element (N) is the size of the matrix,
\item second element (M) is the number of non-zero
elements in the upper part (including) diagonal
\item the remaining (M) elements are the actual
precision (upper triangle plus diagonal) elements
whose order shall follow the graph definition.
}
\item \code{mu} the mean vector,
\item \code{initial} vector with
\itemize{
\item first element as the number of the parameters in the model
\item remaining elements should be the initials for the model parameters.
}
\item \code{log.norm.const} log of the normalizing constant.
\item \code{log.prior} log of the prior for the model parameters.
}
}
\section{Functions}{
\itemize{
\item \code{rgeneric(default)}: The rgeneric default method.

\item \code{rgeneric(rgeneric)}: Returns the model object unchanged.

\item \code{rgeneric(inla.rgeneric)}: Check and converts a regular \code{inla.rgeneric} object to \code{rgeneric}.

\item \code{print(rgeneric)}: Print the rgeneric object

\item \code{summary(rgeneric)}: A summary for a rgeneric object

\item \code{plot(rgeneric)}: A plot for a rgeneric object

}}
\note{
Recommended for prototyping, whereas
\code{cgeneric} is recommended for production.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Matrix.R
\name{Sparse}
\alias{Sparse}
\title{To store in i,j,x sparse matrix format}
\usage{
Sparse(A, unique = TRUE, na.rm = FALSE, zeros.rm = FALSE)
}
\arguments{
\item{A}{matrix or Matrix}

\item{unique}{logical (default is TRUE) to
ensure that the internal representation
is unique and there are no duplicated entries.
(Do not change this unless you know what you are doing.)}

\item{na.rm}{logical (default is FALSE) indicating
if it is to replace ‘NA’'s in the matrix with zeros.}

\item{zeros.rm}{logical (default is FALSE)
indicating if it is to remove zeros in the
matrix. Applied after \code{na.rm}.}
}
\description{
To store in i,j,x sparse matrix format
}
\note{
This is based in INLA::inla.as.sparse(),
but allow all combinations of 'na.rm' and 'zeros.rm'.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kronecker.R
\name{extraconstr}
\alias{extraconstr}
\alias{kronecker_extraconstr}
\title{Kronecker (product) between \code{extraconstr},
implemented for \code{\link[=kronecker]{kronecker()}} methods.}
\usage{
kronecker_extraconstr(c1, c2, n1, n2)
}
\arguments{
\item{c1, c2}{named list with two elements:
\code{A} and \code{e}, where \code{nrow(A)} should be equal
to \code{length(e)}. These are constraint definitions.}

\item{n1, n2}{integer with each model's length.}
}
\value{
The constraint definition for the
whole latent model built from the Kronecker product.
A length two named list. 'A' a matrix and
'e' a vector where nrow(A)=length(e) and
ncol(A)=(n1*n2).
}
\description{
Kronecker (product) between \code{extraconstr},
implemented for \code{\link[=kronecker]{kronecker()}} methods.
}
